# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

__all__ = [
    'LinodeDNSDriver'
]

from datetime import datetime

try:
    import simplejson as json
except ImportError:
    import json

from libcloud.utils.py3 import httplib
from libcloud.utils.misc import merge_valid_keys, get_new_obj
from libcloud.common.linode import (API_ROOT, DEFAULT_API_VERSION,
                                    LinodeException, LinodeConnection,
                                    LinodeResponse,
                                    LinodeExceptionV4, LinodeConnectionV4,
                                    LinodeResponseV4)
from libcloud.dns.types import Provider, RecordType
from libcloud.dns.types import ZoneDoesNotExistError, RecordDoesNotExistError
from libcloud.dns.base import DNSDriver, Zone, Record


VALID_ZONE_EXTRA_PARAMS = ['SOA_Email', 'Refresh_sec', 'Retry_sec',
                           'Expire_sec', 'status', 'master_ips']

VALID_RECORD_EXTRA_PARAMS = ['Priority', 'Weight', 'Port', 'Protocol',
                             'TTL_sec']

VALID_ZONE_EXTRA_PARAMS_V4 = ['description', 'expire_sec', 'master_ips',
                              'refresh_sec', 'retry_sec', 'soa_email',
                              'status', 'tags']

VALID_RECORD_EXTRA_PARAMS_V4 = ['port', 'priority', 'protocol', 'service',
                                'tag', 'ttl_sec', 'target', 'weight']


class LinodeDNSDriver(DNSDriver):
    type = Provider.LINODE
    name = 'Linode DNS'
    website = 'http://www.linode.com/'

    def __new__(cls, key, secret=None, secure=True,
                host=None, port=None,
                api_version=DEFAULT_API_VERSION, **kwargs):

        if cls is LinodeDNSDriver:
            if api_version == '3.0':
                cls = LinodeDNSDriverV3
            elif api_version == '4.0':
                cls = LinodeDNSDriverV4
            else:
                raise NotImplementedError(
                    'No Linode driver found for API version: %s' %
                    (api_version))
        return super(LinodeDNSDriver, cls).__new__(cls)


class LinodeDNSResponse(LinodeResponse):
    def _make_excp(self, error):
        result = super(LinodeDNSResponse, self)._make_excp(error)
        if isinstance(result, LinodeException) and result.code == 5:
            context = self.connection.context

            if context['resource'] == 'zone':
                result = ZoneDoesNotExistError(value='',
                                               driver=self.connection.driver,
                                               zone_id=context['id'])

            elif context['resource'] == 'record':
                result = RecordDoesNotExistError(value='',
                                                 driver=self.connection.driver,
                                                 record_id=context['id'])
        return result


class LinodeDNSConnection(LinodeConnection):
    responseCls = LinodeDNSResponse


class LinodeDNSDriverV3(LinodeDNSDriver):
    connectionCls = LinodeDNSConnection

    RECORD_TYPE_MAP = {
        RecordType.NS: 'NS',
        RecordType.MX: 'MX',
        RecordType.A: 'A',
        RecordType.AAAA: 'AAAA',
        RecordType.CNAME: 'CNAME',
        RecordType.TXT: 'TXT',
        RecordType.SRV: 'SRV',
    }

    def list_zones(self):
        params = {'api_action': 'domain.list'}
        data = self.connection.request(API_ROOT, params=params).objects[0]
        zones = self._to_zones(data)
        return zones

    def list_records(self, zone):
        params = {'api_action': 'domain.resource.list', 'DOMAINID': zone.id}

        self.connection.set_context(context={'resource': 'zone',
                                             'id': zone.id})
        data = self.connection.request(API_ROOT, params=params).objects[0]
        records = self._to_records(items=data, zone=zone)
        return records

    def get_zone(self, zone_id):
        params = {'api_action': 'domain.list', 'DomainID': zone_id}
        self.connection.set_context(context={'resource': 'zone',
                                             'id': zone_id})
        data = self.connection.request(API_ROOT, params=params).objects[0]
        zones = self._to_zones(data)

        if len(zones) != 1:
            raise ZoneDoesNotExistError(value='', driver=self, zone_id=zone_id)

        return zones[0]

    def get_record(self, zone_id, record_id):
        zone = self.get_zone(zone_id=zone_id)
        params = {'api_action': 'domain.resource.list', 'DomainID': zone_id,
                  'ResourceID': record_id}
        self.connection.set_context(context={'resource': 'record',
                                             'id': record_id})
        data = self.connection.request(API_ROOT, params=params).objects[0]
        records = self._to_records(items=data, zone=zone)

        if len(records) != 1:
            raise RecordDoesNotExistError(value='', driver=self,
                                          record_id=record_id)

        return records[0]

    def create_zone(self, domain, type='master', ttl=None, extra=None):
        """
        Create a new zone.

        API docs: http://www.linode.com/api/dns/domain.create
        """
        params = {'api_action': 'domain.create', 'Type': type,
                  'Domain': domain}

        if ttl:
            params['TTL_sec'] = ttl

        merged = merge_valid_keys(params=params,
                                  valid_keys=VALID_ZONE_EXTRA_PARAMS,
                                  extra=extra)
        data = self.connection.request(API_ROOT, params=params).objects[0]
        zone = Zone(id=data['DomainID'], domain=domain, type=type, ttl=ttl,
                    extra=merged, driver=self)
        return zone

    def update_zone(self, zone, domain=None, type=None, ttl=None, extra=None):
        """
        Update an existing zone.

        API docs: http://www.linode.com/api/dns/domain.update
        """
        params = {'api_action': 'domain.update', 'DomainID': zone.id}

        if type:
            params['Type'] = type

        if domain:
            params['Domain'] = domain

        if ttl:
            params['TTL_sec'] = ttl

        merged = merge_valid_keys(params=params,
                                  valid_keys=VALID_ZONE_EXTRA_PARAMS,
                                  extra=extra)
        self.connection.request(API_ROOT, params=params).objects[0]
        updated_zone = get_new_obj(obj=zone, klass=Zone,
                                   attributes={'domain': domain,
                                               'type': type, 'ttl': ttl,
                                               'extra': merged})
        return updated_zone

    def create_record(self, name, zone, type, data, extra=None):
        """
        Create a new record.

        API docs: http://www.linode.com/api/dns/domain.resource.create
        """
        params = {'api_action': 'domain.resource.create', 'DomainID': zone.id,
                  'Name': name, 'Target': data,
                  'Type': self.RECORD_TYPE_MAP[type]}
        merged = merge_valid_keys(params=params,
                                  valid_keys=VALID_RECORD_EXTRA_PARAMS,
                                  extra=extra)

        result = self.connection.request(API_ROOT, params=params).objects[0]
        record = Record(id=result['ResourceID'], name=name, type=type,
                        data=data, extra=merged, zone=zone, driver=self,
                        ttl=merged.get('TTL_sec', None))
        return record

    def update_record(self, record, name=None, type=None, data=None,
                      extra=None):
        """
        Update an existing record.

        API docs: http://www.linode.com/api/dns/domain.resource.update
        """
        params = {'api_action': 'domain.resource.update',
                  'ResourceID': record.id, 'DomainID': record.zone.id}

        if name:
            params['Name'] = name

        if data:
            params['Target'] = data

        if type is not None:
            params['Type'] = self.RECORD_TYPE_MAP[type]

        merged = merge_valid_keys(params=params,
                                  valid_keys=VALID_RECORD_EXTRA_PARAMS,
                                  extra=extra)

        self.connection.request(API_ROOT, params=params).objects[0]
        updated_record = get_new_obj(obj=record, klass=Record,
                                     attributes={'name': name, 'data': data,
                                                 'type': type,
                                                 'extra': merged})
        return updated_record

    def delete_zone(self, zone):
        params = {'api_action': 'domain.delete', 'DomainID': zone.id}

        self.connection.set_context(context={'resource': 'zone',
                                             'id': zone.id})
        data = self.connection.request(API_ROOT, params=params).objects[0]

        return 'DomainID' in data

    def delete_record(self, record):
        params = {'api_action': 'domain.resource.delete',
                  'DomainID': record.zone.id, 'ResourceID': record.id}

        self.connection.set_context(context={'resource': 'record',
                                             'id': record.id})
        data = self.connection.request(API_ROOT, params=params).objects[0]

        return 'ResourceID' in data

    def _to_zones(self, items):
        """
        Convert a list of items to the Zone objects.
        """
        zones = []

        for item in items:
            zones.append(self._to_zone(item))

        return zones

    def _to_zone(self, item):
        """
        Build an Zone object from the item dictionary.
        """
        extra = {'SOA_Email': item['SOA_EMAIL'], 'status': item['STATUS'],
                 'description': item['DESCRIPTION']}
        zone = Zone(id=item['DOMAINID'], domain=item['DOMAIN'],
                    type=item['TYPE'], ttl=item['TTL_SEC'], driver=self,
                    extra=extra)
        return zone

    def _to_records(self, items, zone=None):
        """
        Convert a list of items to the Record objects.
        """
        records = []

        for item in items:
            records.append(self._to_record(item=item, zone=zone))

        return records

    def _to_record(self, item, zone=None):
        """
        Build a Record object from the item dictionary.
        """
        extra = {'protocol': item['PROTOCOL'], 'ttl_sec': item['TTL_SEC'],
                 'port': item['PORT'], 'weight': item['WEIGHT'],
                 'priority': item['PRIORITY']}
        type = self._string_to_record_type(item['TYPE'])
        record = Record(id=item['RESOURCEID'], name=item['NAME'], type=type,
                        data=item['TARGET'], zone=zone, driver=self,
                        ttl=item['TTL_SEC'], extra=extra)
        return record


class LinodeDNSResponseV4(LinodeResponseV4):
    pass


class LinodeDNSConnectionV4(LinodeConnectionV4):
    responseCls = LinodeDNSResponseV4


class LinodeDNSDriverV4(LinodeDNSDriver):
    connectionCls = LinodeDNSConnectionV4

    RECORD_TYPE_MAP = {
        RecordType.SOA: 'SOA',
        RecordType.NS: 'NS',
        RecordType.MX: 'MX',
        RecordType.A: 'A',
        RecordType.AAAA: 'AAAA',
        RecordType.CNAME: 'CNAME',
        RecordType.TXT: 'TXT',
        RecordType.SRV: 'SRV',
        RecordType.CAA: 'CAA',
    }

    def list_zones(self):
        """
        Return a list of zones.

        :return: ``list`` of :class:`Zone`
        """
        data = self._paginated_request('/v4/domains', 'data')
        return [self._to_zone(zone) for zone in data]

    def list_records(self, zone):
        """
        Return a list of records for the provided zone.

        :param zone: Zone to list records for.
        :type zone: :class:`Zone`

        :return: ``list`` of :class:`Record`
        """
        if not isinstance(zone, Zone):
            raise LinodeExceptionV4("Invalid zone instance")

        data = self._paginated_request('/v4/domains/%s/records'
                                       % zone.id, 'data')

        return [self._to_record(record, zone=zone) for record in data]

    def get_zone(self, zone_id):
        """
        Return a Zone instance.

        :param zone_id: ID of the zone to get
        :type  zone_id: ``str``

        :rtype: :class:`Zone`
        """
        data = self.connection.request('/v4/domains/%s' %
                                       zone_id).object

        return self._to_zone(data)

    def get_record(self, zone_id, record_id):
        """
        Return a record instance.

        :param zone_id: ID of the record's zone
        :type  zone_id: ``str``

        :param record_id: ID of the record to get
        :type  record_id: ``str``

        :rtype: :class:`Record`
        """
        data = self.connection.request('/v4/domains/%s/records/%s' %
                                       (zone_id, record_id)).object

        return self._to_record(data, self.get_zone(zone_id))

    def create_zone(self, domain, type='master', ttl=None, extra=None):
        """
        Create a new zone.

        :param domain: Zone domain name (e.g. example.com)
        :type domain: ``str``

        :keyword type: Zone type (master / slave).
        :type  type: ``str``

        :keyword ttl: TTL for new records. (optional)
        :type  ttl: ``int``

        :keyword extra: Extra attributes.('description', 'expire_sec', \
        'master_ips','refresh_sec', 'retry_sec', 'soa_email',\
        'status', 'tags'). 'soa_email' required for master zones
        :type extra: ``dict``

        :rtype: :class:`Zone`
        """

        attr = {
            'domain': domain,
            'type': type,
        }
        if ttl is not None:
            attr['ttl_sec'] = ttl

        merge_valid_keys(params=attr,
                         valid_keys=VALID_ZONE_EXTRA_PARAMS_V4,
                         extra=extra)
        data = self.connection.request(
            '/v4/domains', data=json.dumps(attr),
            method='POST').object

        return self._to_zone(data)

    def create_record(self, name, zone, type, data, extra=None):
        """
        Create a record.

        :param name: The name of this Record.For A records\
        this is the subdomain being associated with an IP address.
        :type  name: ``str``

        :param zone: Zone which the records will be created for.
        :type zone: :class:`Zone`

        :param type: DNS record type.
        :type  type: :class:`RecordType`

        :param data: Data for the record (depends on the record type).\
         For A records, this is the address the domain should resolve to.
        :type  data: ``str``

        :param extra: Extra attributes.('port', 'priority', \
        'protocol', 'service', 'tag', 'ttl_sec', 'target', 'weight')
        :type  extra: ``dict``

        :rtype: :class:`Record`
        """
        if not isinstance(zone, Zone):
            raise LinodeExceptionV4("Invalid zone instance")

        attr = {
            'type': self.RECORD_TYPE_MAP[type],
            'name': name,
            'target': data,
        }

        merge_valid_keys(params=attr,
                         valid_keys=VALID_RECORD_EXTRA_PARAMS_V4,
                         extra=extra)
        data = self.connection.request('/v4/domains/%s/records' % zone.id,
                                       data=json.dumps(attr),
                                       method='POST').object

        return self._to_record(data, zone=zone)

    def update_zone(self, zone, domain, type='master', ttl=None, extra=None):
        """
        Update an existing zone.

        :param zone: Zone to update.
        :type  zone: :class:`Zone`

        :param domain: Name of zone
        :type  domain: ``str``

        :param type: Zone type (master / slave).
        :type  type: ``str``

        :param ttl: TTL for new records. (optional)
        :type  ttl: ``int``

        :param extra: Extra attributes ('description', 'expire_sec', \
        'master_ips','refresh_sec', 'retry_sec', 'soa_email','status', 'tags')

        :type  extra: ``dict``

        :rtype: :class:`Zone`
        """
        if not isinstance(zone, Zone):
            raise LinodeExceptionV4("Invalid zone instance")

        attr = {
            'domain': domain,
            'type': type,
        }

        if ttl is not None:
            attr['ttl_sec'] = ttl

        merge_valid_keys(params=attr,
                         valid_keys=VALID_ZONE_EXTRA_PARAMS_V4,
                         extra=extra)

        data = self.connection.request('/v4/domains/%s' % zone.id,
                                       data=json.dumps(attr),
                                       method='PUT').object

        return self._to_zone(data)

    def update_record(self, record, name=None, type=None,
                      data=None, extra=None):
        """
        Update an existing record.

        :param record: Record to update.
        :type  record: :class:`Record`

        :param name: Record name.This field's actual usage\
         depends on the type of record this represents.
        :type  name: ``str``

        :param type: DNS record type
        :type  type: :class:`RecordType`

        :param data: Data for the record (depends on the record type).
        :type  data: ``str``

        :param extra: Extra attributes.('port', 'priority', \
        'protocol', 'service', 'tag', 'ttl_sec', 'target', 'weight')
        :type  extra: ``dict``

        :rtype: :class:`Record`
        """
        if not isinstance(record, Record):
            raise LinodeExceptionV4("Invalid zone instance")

        zone = record.zone
        attr = {}

        if name is not None:
            attr['name'] = name

        if type is not None:
            attr['type'] = self.RECORD_TYPE_MAP[type]

        if data is not None:
            attr['target'] = data

        merge_valid_keys(params=attr,
                         valid_keys=VALID_RECORD_EXTRA_PARAMS_V4,
                         extra=extra)

        data = self.connection.request('/v4/domains/%s/records/%s'
                                       % (zone.id, record.id),
                                       data=json.dumps(attr),
                                       method='PUT').object

        return self._to_record(data, zone=zone)

    def delete_zone(self, zone):
        """
        Delete a zone.

        This will delete all the records belonging to this zone.

        :param zone: Zone to delete.
        :type  zone: :class:`Zone`

        :rtype: ``bool``
        """
        if not isinstance(zone, Zone):
            raise LinodeExceptionV4("Invalid zone instance")

        response = self.connection.request('/v4/domains/%s'
                                           % zone.id,
                                           method='DELETE')
        return response.status == httplib.OK

    def delete_record(self, record):
        """
        Delete a record.

        :param record: Record to delete.
        :type  record: :class:`Record`

        :rtype: ``bool``
        """
        if not isinstance(record, Record):
            raise LinodeExceptionV4("Invalid record instance")

        zone = record.zone

        response = self.connection.request('/v4/domains/%s/records/%s'
                                           % (zone.id, record.id),
                                           method='DELETE')
        return response.status == httplib.OK

    def _to_record(self, item, zone=None):
        extra = {
            'port': item['port'],
            'weight': item['weight'],
            'priority': item['priority'],
            'service': item['service'],
            'protocol': item['protocol'],
            'created': self._to_datetime(item['created']),
            'updated': self._to_datetime(item['updated'])
        }
        type = self._string_to_record_type(item['type'])
        record = Record(id=item['id'], name=item['name'], type=type,
                        data=item['target'], zone=zone, driver=self,
                        ttl=item['ttl_sec'], extra=extra)
        return record

    def _to_zone(self, item):
        """
        Build an Zone object from the item dictionary.
        """
        extra = {'soa_email': item['soa_email'],
                 'status': item['status'],
                 'description': item['description'],
                 'tags': item['tags'],
                 'retry_sec': item['retry_sec'],
                 'master_ips': item['master_ips'],
                 'axfr_ips': item['axfr_ips'],
                 'expire_sec': item['expire_sec'],
                 'refresh_sec': item['refresh_sec'],
                 'created': self._to_datetime(item['created']),
                 'updated': self._to_datetime(item['updated']),
                 }
        zone = Zone(id=item['id'], domain=item['domain'],
                    type=item['type'], ttl=item['ttl_sec'], driver=self,
                    extra=extra)
        return zone

    def _to_datetime(self, strtime):
        return datetime.strptime(strtime, "%Y-%m-%dT%H:%M:%S")

    def _paginated_request(self, url, obj, params=None):
        """
        Perform multiple calls in order to have a full list of elements when
        the API responses are paginated.

        :param url: API endpoint
        :type url: ``str``

        :param obj: Result object key
        :type obj: ``str``

        :param params: Request parameters
        :type params: ``dict``

        :return: ``list`` of API response objects
        :rtype: ``list``
        """
        objects = []
        params = params if params is not None else {}

        ret = self.connection.request(url, params=params).object

        data = list(ret.get(obj, []))
        current_page = int(ret.get('page', 1))
        num_of_pages = int(ret.get('pages', 1))
        objects.extend(data)
        for page in range(current_page + 1, num_of_pages + 1):
            # add param to request next page
            params['page'] = page
            ret = self.connection.request(url, params=params).object
            data = list(ret.get(obj, []))
            objects.extend(data)
        return objects



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import with_statement

from typing import Optional
from typing import List

from libcloud.common.base import ConnectionUserAndKey, BaseDriver
from libcloud.container.types import ContainerState


__all__ = [
    'Container',
    'ContainerImage',
    'ContainerCluster',
    'ClusterLocation',
    'ContainerDriver'
]


class Container(object):
    """
    Container.
    """

    def __init__(self,
                 id,  # type: str
                 name,  # type: str
                 image,  # type: ContainerImage
                 state,  # type: ContainerState
                 ip_addresses,  # type: List[str]
                 driver,  # type: ContainerDriver
                 extra=None  # type: dict
                 ):
        """
        :param id: Container id.
        :type id: ``str``

        :param name: The name of the container.
        :type  name: ``str``

        :param image: The image this container was deployed using.
        :type  image: :class:`.ContainerImage`

        :param state: The state of the container, e.g. running
        :type  state: :class:`libcloud.container.types.ContainerState`

        :param ip_addresses: A list of IP addresses for this container
        :type  ip_addresses: ``list`` of ``str``

        :param driver: ContainerDriver instance.
        :type driver: :class:`.ContainerDriver`

        :param extra: (optional) Extra attributes (driver specific).
        :type extra: ``dict``
        """
        self.id = str(id) if id else None
        self.name = name
        self.image = image
        self.state = state
        self.ip_addresses = ip_addresses
        self.driver = driver
        self.extra = extra or {}

    def start(self):
        # type: () -> Container
        return self.driver.start_container(container=self)

    def stop(self):
        # type: () -> Container
        return self.driver.stop_container(container=self)

    def restart(self):
        # type: () -> Container
        return self.driver.restart_container(container=self)

    def destroy(self):
        # type: () -> bool
        return self.driver.destroy_container(container=self)

    def __repr__(self):
        return ('<Container: id=%s, name=%s,'
                'state=%s, provider=%s ...>' %
                (self.id, self.name, self.state,
                 self.driver.name))


class ContainerImage(object):
    """
    Container Image.
    """

    def __init__(self,
                 id,  # type: str
                 name,  # type: str
                 path,  # type: str
                 version,  # type: str
                 driver,  # type: ContainerDriver
                 extra=None  # type: dict
                 ):
        """
        :param id: Container Image id.
        :type id: ``str``

        :param name: The name of the image.
        :type  name: ``str``

        :param path: The path to the image
        :type  path: ``str``

        :param version: The version of the image
        :type  version: ``str``

        :param driver: ContainerDriver instance.
        :type driver: :class:`.ContainerDriver`

        :param extra: (optional) Extra attributes (driver specific).
        :type extra: ``dict``
        """
        self.id = str(id) if id else None
        self.name = name
        self.path = path
        self.version = version
        self.driver = driver
        self.extra = extra or {}

    def deploy(self, name, parameters, cluster=None, start=True):
        # type: (str, str, Optional[ContainerCluster], bool) -> Container
        return self.driver.deploy_container(name=name,
                                            image=self,
                                            parameters=parameters,
                                            cluster=cluster,
                                            start=start)

    def __repr__(self):
        return ('<ContainerImage: id=%s, name=%s, path=%s ...>' %
                (self.id, self.name, self.path))


class ContainerCluster(object):
    """
    A cluster group for containers
    """

    def __init__(self,
                 id,  # type: str
                 name,  # type: str
                 driver,  # type: ContainerDriver
                 extra=None  # type: dict
                 ):
        """
        :param id: Container Image id.
        :type id: ``str``

        :param name: The name of the image.
        :type  name: ``str``

        :param driver: ContainerDriver instance.
        :type driver: :class:`.ContainerDriver`

        :param extra: (optional) Extra attributes (driver specific).
        :type extra: ``dict``
        """
        self.id = str(id) if id else None
        self.name = name
        self.driver = driver
        self.extra = extra or {}

    def list_containers(self):
        # type: () -> List[Container]
        return self.driver.list_containers(cluster=self)

    def destroy(self):
        # type: () -> bool
        return self.driver.destroy_cluster(cluster=self)

    def __repr__(self):
        return ('<ContainerCluster: id=%s, name=%s, provider=%s ...>' %
                (self.id, self.name, self.driver.name))


class ClusterLocation(object):
    """
    A physical location where clusters can be.

    >>> from libcloud.container.drivers.dummy import DummyContainerDriver
    >>> driver = DummyContainerDriver(0)
    >>> location = driver.list_locations()[0]
    >>> location.country
    'US'
    """

    def __init__(self,
                 id,  # type: str
                 name,  # type: str
                 country,  # type: str
                 driver  # type: ContainerDriver
                 ):
        """
        :param id: Location ID.
        :type id: ``str``

        :param name: Location name.
        :type name: ``str``

        :param country: Location country.
        :type country: ``str``

        :param driver: Driver this location belongs to.
        :type driver: :class:`.ContainerDriver`
        """
        self.id = str(id)
        self.name = name
        self.country = country
        self.driver = driver

    def __repr__(self):
        return (('<ClusterLocation: id=%s, name=%s, country=%s, driver=%s>')
                % (self.id, self.name, self.country, self.driver.name))


class ContainerDriver(BaseDriver):
    """
    A base ContainerDriver class to derive from

    This class is always subclassed by a specific driver.
    """
    connectionCls = ConnectionUserAndKey
    name = None
    website = None
    supports_clusters = False
    """
    Whether the driver supports containers being deployed into clusters
    """

    def __init__(self, key, secret=None, secure=True, host=None, port=None,
                 **kwargs):
        """
        :param    key: API key or username to used (required)
        :type     key: ``str``

        :param    secret: Secret password to be used (required)
        :type     secret: ``str``

        :param    secure: Whether to use HTTPS or HTTP. Note: Some providers
                only support HTTPS, and it is on by default.
        :type     secure: ``bool``

        :param    host: Override hostname used for connections.
        :type     host: ``str``

        :param    port: Override port used for connections.
        :type     port: ``int``

        :return: ``None``
        """
        super(ContainerDriver, self).__init__(
            key=key, secret=secret, secure=secure,
            host=host, port=port, **kwargs)

    def install_image(self, path):
        # type: (str) -> ContainerImage
        """
        Install a container image from a remote path.

        :param path: Path to the container image
        :type  path: ``str``

        :rtype: :class:`.ContainerImage`
        """
        raise NotImplementedError(
            'install_image not implemented for this driver')

    def list_images(self):
        # type: () -> List[ContainerImage]
        """
        List the installed container images

        :rtype: ``list`` of :class:`.ContainerImage`
        """
        raise NotImplementedError(
            'list_images not implemented for this driver')

    def list_containers(self,
                        image=None,  # type: Optional[ContainerImage]
                        cluster=None  # type: Optional[ContainerCluster]
                        ):
        # type: (...) -> List[Container]
        """
        List the deployed container images

        :param image: Filter to containers with a certain image
        :type  image: :class:`.ContainerImage`

        :param cluster: Filter to containers in a cluster
        :type  cluster: :class:`.ContainerCluster`

        :rtype: ``list`` of :class:`.Container`
        """
        raise NotImplementedError(
            'list_containers not implemented for this driver')

    def deploy_container(self,
                         name,  # type: str
                         image,  # type: ContainerImage
                         cluster=None,  # type: Optional[ContainerCluster]
                         parameters=None,  # type: Optional[str]
                         start=True  # type: bool
                         ):
        # type: (...) -> Container
        """
        Deploy an installed container image

        :param name: The name of the new container
        :type  name: ``str``

        :param image: The container image to deploy
        :type  image: :class:`.ContainerImage`

        :param cluster: The cluster to deploy to, None is default
        :type  cluster: :class:`.ContainerCluster`

        :param parameters: Container Image parameters
        :type  parameters: ``str``

        :param start: Start the container on deployment
        :type  start: ``bool``

        :rtype: :class:`.Container`
        """
        raise NotImplementedError(
            'deploy_container not implemented for this driver')

    def get_container(self, id):
        # type: (str) -> Container
        """
        Get a container by ID

        :param id: The ID of the container to get
        :type  id: ``str``

        :rtype: :class:`.Container`
        """
        raise NotImplementedError(
            'get_container not implemented for this driver')

    def start_container(self, container):
        # type: (Container) -> Container
        """
        Start a deployed container

        :param container: The container to start
        :type  container: :class:`.Container`

        :rtype: :class:`.Container`
        """
        raise NotImplementedError(
            'start_container not implemented for this driver')

    def stop_container(self, container):
        # type: (Container) -> Container
        """
        Stop a deployed container

        :param container: The container to stop
        :type  container: :class:`.Container`

        :rtype: :class:`.Container`
        """
        raise NotImplementedError(
            'stop_container not implemented for this driver')

    def restart_container(self, container):
        # type: (Container) -> Container
        """
        Restart a deployed container

        :param container: The container to restart
        :type  container: :class:`.Container`

        :rtype: :class:`.Container`
        """
        raise NotImplementedError(
            'restart_container not implemented for this driver')

    def destroy_container(self, container):
        # type: (Container) -> bool
        """
        Destroy a deployed container

        :param container: The container to destroy
        :type  container: :class:`.Container`

        :rtype: ``bool``
        """
        raise NotImplementedError(
            'destroy_container not implemented for this driver')

    def list_locations(self):
        # type: () -> List[ClusterLocation]
        """
        Get a list of potential locations to deploy clusters into

        :rtype: ``list`` of :class:`.ClusterLocation`
        """
        raise NotImplementedError(
            'list_locations not implemented for this driver')

    def create_cluster(self, name, location=None):
        # type: (str, Optional[ClusterLocation]) -> ContainerCluster
        """
        Create a container cluster

        :param  name: The name of the cluster
        :type   name: ``str``

        :param  location: The location to create the cluster in
        :type   location: :class:`.ClusterLocation`

        :rtype: :class:`.ContainerCluster`
        """
        raise NotImplementedError(
            'create_cluster not implemented for this driver')

    def destroy_cluster(self, cluster):
        # type: (ContainerCluster) -> bool
        """
        Delete a cluster

        :return: ``True`` if the destroy was successful, otherwise ``False``.
        :rtype: ``bool``
        """
        raise NotImplementedError(
            'destroy_cluster not implemented for this driver')

    def list_clusters(self, location=None):
        # type: (Optional[ClusterLocation]) -> List[ContainerCluster]
        """
        Get a list of potential locations to deploy clusters into

        :param  location: The location to search in
        :type   location: :class:`.ClusterLocation`

        :rtype: ``list`` of :class:`.ContainerCluster`
        """
        raise NotImplementedError(
            'list_clusters not implemented for this driver')

    def get_cluster(self, id):
        # type: (str) -> ContainerCluster
        """
        Get a cluster by ID

        :param id: The ID of the cluster to get
        :type  id: ``str``

        :rtype: :class:`.ContainerCluster`
        """
        raise NotImplementedError(
            'list_clusters not implemented for this driver')



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import List
from typing import Dict

from libcloud.common.base import ConnectionKey, JsonResponse


__all__ = [
    'API_HOST',
    'DNSPodException',
    'DNSPodResponse',
    'DNSPodConnection'
]

# Endpoint for dnspod api
API_HOST = 'api.dnspod.com'


class DNSPodResponse(JsonResponse):
    errors = []  # type: List[Dict]
    objects = []  # type: List[Dict]

    def __init__(self, response, connection):
        super(DNSPodResponse, self).__init__(response=response,
                                             connection=connection)
        self.errors, self.objects = self.parse_body_and_errors()
        if not self.success():
            raise DNSPodException(code=self.status,
                                  message=self.errors.pop()
                                  ['status']['message'])

    def parse_body_and_errors(self):
        js = super(DNSPodResponse, self).parse_body()
        if 'status' in js and js['status']['code'] != '1':
            self.errors.append(js)
        else:
            self.objects.append(js)

        return self.errors, self.objects

    def success(self):
        return len(self.errors) == 0


class DNSPodConnection(ConnectionKey):
    host = API_HOST
    responseCls = DNSPodResponse

    def add_default_headers(self, headers):
        headers['Content-Type'] = 'application/x-www-form-urlencoded'
        headers['Accept'] = 'text/json'

        return headers


class DNSPodException(Exception):

    def __init__(self, code, message):
        self.code = code
        self.message = message
        self.args = (code, message)

    def __str__(self):
        return "%s %s" % (self.code, self.message)

    def __repr__(self):
        return "DNSPodException %s %s" % (self.code, self.message)



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

from libcloud.compute.drivers.kili import KiliCloudNodeDriver, ENDPOINT_ARGS
from libcloud.test.compute.test_openstack import OpenStack_1_1_Tests


def _ex_connection_class_kwargs(self):
    kwargs = self.openstack_connection_kwargs()
    kwargs['get_endpoint_args'] = ENDPOINT_ARGS
    # Remove keystone from the URL path so that the openstack base tests work
    kwargs['ex_force_auth_url'] = 'https://api.kili.io/v2.0/tokens'
    kwargs['ex_tenant_name'] = self.tenant_name

    return kwargs

KiliCloudNodeDriver._ex_connection_class_kwargs = _ex_connection_class_kwargs


class KiliCloudNodeDriverTests(OpenStack_1_1_Tests, unittest.TestCase):
    driver_klass = KiliCloudNodeDriver
    driver_type = KiliCloudNodeDriver



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and

import sys
import unittest

from libcloud.dns.drivers.digitalocean import DigitalOceanDNSDriver
from libcloud.dns.types import RecordType
from libcloud.test import LibcloudTestCase, MockHttp
from libcloud.test.file_fixtures import DNSFileFixtures
from libcloud.test.secrets import DIGITALOCEAN_v2_PARAMS
from libcloud.utils.py3 import httplib


class DigitalOceanDNSTests(LibcloudTestCase):

    def setUp(self):
        DigitalOceanDNSDriver.connectionCls.conn_class = DigitalOceanDNSMockHttp
        DigitalOceanDNSMockHttp.type = None
        self.driver = DigitalOceanDNSDriver(*DIGITALOCEAN_v2_PARAMS)

    def test_list_zones(self):
        zones = self.driver.list_zones()
        self.assertTrue(len(zones) >= 1)

    def test_get_zone(self):
        zone = self.driver.get_zone('testdomain')
        self.assertEqual(zone.id, 'testdomain')

    def test_get_zone_not_found(self):
        DigitalOceanDNSMockHttp.type = 'NOT_FOUND'
        self.assertRaises(Exception, self.driver.get_zone, 'testdomain')

    def test_list_records(self):
        zone = self.driver.get_zone('testdomain')
        records = self.driver.list_records(zone)
        self.assertTrue(len(records) >= 1)
        self.assertEqual(records[1].ttl, 1800)
        self.assertEqual(records[4].ttl, None)

    def test_get_record(self):
        record = self.driver.get_record('testdomain', '1234564')
        self.assertEqual(record.id, '1234564')
        self.assertEqual(record.type, RecordType.A)
        self.assertEqual(record.data, '123.45.67.89')
        self.assertEqual(record.ttl, 1800)

    def test_get_record_not_found(self):
        DigitalOceanDNSMockHttp.type = 'NOT_FOUND'
        self.assertRaises(Exception, self.driver.get_zone, 'testdomain')

    def test_create_zone(self):
        DigitalOceanDNSMockHttp.type = 'CREATE'
        zone = self.driver.create_zone('testdomain')
        self.assertEqual(zone.id, 'testdomain')

    def test_create_record(self):
        zone = self.driver.get_zone('testdomain')

        DigitalOceanDNSMockHttp.type = 'CREATE'
        record = self.driver.create_record('sub', zone,
                                           RecordType.A, '234.56.78.90',
                                           extra={'ttl': 60})
        self.assertEqual(record.id, '1234565')
        self.assertEqual(record.type, RecordType.A)
        self.assertEqual(record.data, '234.56.78.90')
        self.assertEqual(record.ttl, 60)

    def test_update_record(self):
        record = self.driver.get_record('testdomain', '1234564')

        DigitalOceanDNSMockHttp.type = 'UPDATE'
        record = self.driver.update_record(record, data="234.56.78.90",
                                           extra={'ttl': 60})
        self.assertEqual(record.id, '1234564')
        self.assertEqual(record.data, "234.56.78.90")
        self.assertEqual(record.ttl, 60)

    def test_delete_zone(self):
        zone = self.driver.get_zone('testdomain')

        DigitalOceanDNSMockHttp.type = 'DELETE'
        self.assertTrue(self.driver.delete_zone(zone))

    def test_delete_record(self):
        record = self.driver.get_record('testdomain', '1234564')

        DigitalOceanDNSMockHttp.type = 'DELETE'
        self.assertTrue(self.driver.delete_record(record))


class DigitalOceanDNSMockHttp(MockHttp):
    fixtures = DNSFileFixtures('digitalocean')

    response_map = {
        None: httplib.OK,
        'CREATE': httplib.CREATED,
        'DELETE': httplib.NO_CONTENT,
        'EMPTY': httplib.OK,
        'NOT_FOUND': httplib.NOT_FOUND,
        'UNAUTHORIZED': httplib.UNAUTHORIZED,
        'UPDATE': httplib.OK,
        'UNPROCESSABLE': httplib.UNPROCESSABLE_ENTITY,
    }

    def _v2_domains(self, method, url, body, headers):
        body = self.fixtures.load('_v2_domains.json')
        return (self.response_map[self.type], body, {},
                httplib.responses[self.response_map[self.type]])

    def _v2_domains_CREATE(self, method, url, body, headers):
        if body is None:
            body = self.fixtures.load('_v2_domains_UNPROCESSABLE_ENTITY.json')
            return (self.response_map['UNPROCESSABLE'], body, {},
                    httplib.responses[self.response_map['UNPROCESSABLE']])
        body = self.fixtures.load('_v2_domains_CREATE.json')
        return (self.response_map[self.type], body, {},
                httplib.responses[self.response_map[self.type]])

    def _v2_domains_testdomain(self, method, url, body, headers):
        body = self.fixtures.load('_v2_domains_testdomain.json')
        return (self.response_map[self.type], body, {},
                httplib.responses[self.response_map[self.type]])

    def _v2_domains_testdomain_DELETE(self, method, url, body, headers):
        return (self.response_map[self.type], body, {},
                httplib.responses[self.response_map[self.type]])

    def _v2_domains_testdomain_NOT_FOUND(self, method, url, body, headers):
        body = self.fixtures.load('_v2_domains_testdomain_NOT_FOUND.json')
        return (self.response_map[self.type], body, {},
                httplib.responses[self.response_map[self.type]])

    def _v2_domains_testdomain_records(self, method, url, body, headers):
        body = self.fixtures.load('_v2_domains_testdomain_records.json')
        return (self.response_map[self.type], body, {},
                httplib.responses[self.response_map[self.type]])

    def _v2_domains_testdomain_records_CREATE(self, method, url,
                                              body, headers):
        if body is None:
            body = self.fixtures.load('_v2_domains_UNPROCESSABLE_ENTITY.json')
            return (self.response_map['UNPROCESSABLE'], body, {},
                    httplib.responses[self.response_map['UNPROCESSABLE']])
        body = self.fixtures.load('_v2_domains_testdomain_records_CREATE.json')
        return (self.response_map[self.type], body, {},
                httplib.responses[self.response_map[self.type]])

    def _v2_domains_testdomain_records_1234564(
            self, method, url, body, headers):
        body = self.fixtures.load(
            '_v2_domains_testdomain_records_1234564.json')
        return (self.response_map[self.type], body, {},
                httplib.responses[self.response_map[self.type]])

    def _v2_domains_testdomain_records_1234564_DELETE(
            self, method, url, body, headers):
        self.type = 'DELETE'
        return (self.response_map[self.type], body, {},
                httplib.responses[self.response_map[self.type]])

    def _v2_domains_testdomain_records_1234564_UPDATE(
            self, method, url, body, headers):
        if body is None:
            body = self.fixtures.load('_v2_domains_UNPROCESSABLE_ENTITY.json')
            return (self.response_map['UNPROCESSABLE'], body, {},
                    httplib.responses[self.response_map['UNPROCESSABLE']])
        body = self.fixtures.load(
            '_v2_domains_testdomain_records_1234564_UPDATE.json')
        return (self.response_map[self.type], body, {},
                httplib.responses[self.response_map[self.type]])

if __name__ == '__main__':
    sys.exit(unittest.main())



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Type
from typing import Union
from types import ModuleType
from typing import TYPE_CHECKING

from libcloud.common.providers import get_driver as _get_provider_driver
from libcloud.common.providers import set_driver as _set_provider_driver
from libcloud.storage.types import OLD_CONSTANT_TO_NEW_MAPPING
from libcloud.storage.types import Provider

if TYPE_CHECKING:
    from libcloud.storage.base import StorageDriver

DRIVERS = {
    Provider.DUMMY:
    ('libcloud.storage.drivers.dummy', 'DummyStorageDriver'),
    Provider.CLOUDFILES:
    ('libcloud.storage.drivers.cloudfiles', 'CloudFilesStorageDriver'),
    Provider.OPENSTACK_SWIFT:
    ('libcloud.storage.drivers.cloudfiles', 'OpenStackSwiftStorageDriver'),
    Provider.S3:
    ('libcloud.storage.drivers.s3', 'S3StorageDriver'),
    Provider.S3_US_EAST2:
    ('libcloud.storage.drivers.s3', 'S3USEast2StorageDriver'),
    Provider.S3_US_WEST:
    ('libcloud.storage.drivers.s3', 'S3USWestStorageDriver'),
    Provider.S3_US_WEST_OREGON:
    ('libcloud.storage.drivers.s3', 'S3USWestOregonStorageDriver'),
    Provider.S3_US_GOV_WEST:
    ('libcloud.storage.drivers.s3', 'S3USGovWestStorageDriver'),
    Provider.S3_CN_NORTH:
    ('libcloud.storage.drivers.s3', 'S3CNNorthStorageDriver'),
    Provider.S3_CN_NORTHWEST:
    ('libcloud.storage.drivers.s3', 'S3CNNorthWestStorageDriver'),
    Provider.S3_EU_WEST:
    ('libcloud.storage.drivers.s3', 'S3EUWestStorageDriver'),
    Provider.S3_EU_WEST2:
    ('libcloud.storage.drivers.s3', 'S3EUWest2StorageDriver'),
    Provider.S3_EU_CENTRAL:
    ('libcloud.storage.drivers.s3', 'S3EUCentralStorageDriver'),
    Provider.S3_EU_NORTH1:
    ('libcloud.storage.drivers.s3', 'S3EUNorth1StorageDriver'),
    Provider.S3_AP_SOUTH:
    ('libcloud.storage.drivers.s3', 'S3APSouthStorageDriver'),
    Provider.S3_AP_SOUTHEAST:
    ('libcloud.storage.drivers.s3', 'S3APSEStorageDriver'),
    Provider.S3_AP_SOUTHEAST2:
    ('libcloud.storage.drivers.s3', 'S3APSE2StorageDriver'),
    Provider.S3_AP_NORTHEAST:
    ('libcloud.storage.drivers.s3', 'S3APNE1StorageDriver'),
    Provider.S3_AP_NORTHEAST1:
    ('libcloud.storage.drivers.s3', 'S3APNE1StorageDriver'),
    Provider.S3_AP_NORTHEAST2:
    ('libcloud.storage.drivers.s3', 'S3APNE2StorageDriver'),
    Provider.S3_SA_EAST:
    ('libcloud.storage.drivers.s3', 'S3SAEastStorageDriver'),
    Provider.S3_CA_CENTRAL:
    ('libcloud.storage.drivers.s3', 'S3CACentralStorageDriver'),
    Provider.S3_RGW:
    ('libcloud.storage.drivers.rgw', 'S3RGWStorageDriver'),
    Provider.S3_RGW_OUTSCALE:
    ('libcloud.storage.drivers.rgw', 'S3RGWOutscaleStorageDriver'),
    Provider.NINEFOLD:
    ('libcloud.storage.drivers.ninefold', 'NinefoldStorageDriver'),
    Provider.GOOGLE_STORAGE:
    ('libcloud.storage.drivers.google_storage', 'GoogleStorageDriver'),
    Provider.NIMBUS:
    ('libcloud.storage.drivers.nimbus', 'NimbusStorageDriver'),
    Provider.LOCAL:
    ('libcloud.storage.drivers.local', 'LocalStorageDriver'),
    Provider.AZURE_BLOBS:
    ('libcloud.storage.drivers.azure_blobs', 'AzureBlobsStorageDriver'),
    Provider.KTUCLOUD:
    ('libcloud.storage.drivers.ktucloud', 'KTUCloudStorageDriver'),
    Provider.AURORAOBJECTS:
    ('libcloud.storage.drivers.auroraobjects', 'AuroraObjectsStorageDriver'),
    Provider.BACKBLAZE_B2:
    ('libcloud.storage.drivers.backblaze_b2', 'BackblazeB2StorageDriver'),
    Provider.ALIYUN_OSS:
    ('libcloud.storage.drivers.oss', 'OSSStorageDriver'),
    Provider.DIGITALOCEAN_SPACES:
    ('libcloud.storage.drivers.digitalocean_spaces',
     'DigitalOceanSpacesStorageDriver'),
    Provider.MINIO:
    ('libcloud.storage.drivers.minio', 'MinIOStorageDriver'),
}


def get_driver(provider):
    # type: (Union[Provider, str]) -> Type[StorageDriver]
    deprecated_constants = OLD_CONSTANT_TO_NEW_MAPPING
    return _get_provider_driver(drivers=DRIVERS, provider=provider,
                                deprecated_constants=deprecated_constants)


def set_driver(provider, module, klass):
    # type: (Union[Provider, str], ModuleType, type) -> Type[StorageDriver]
    return _set_provider_driver(drivers=DRIVERS, provider=provider,
                                module=module, klass=klass)



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import unittest

from libcloud.utils.py3 import httplib
from libcloud.utils.py3 import urlparse
from libcloud.utils.py3 import parse_qsl

try:
    import simplejson as json
except ImportError:
    import json

from libcloud.compute.drivers.ktucloud import KTUCloudNodeDriver

from libcloud.test import MockHttp
from libcloud.test.compute import TestCaseMixin
from libcloud.test.file_fixtures import ComputeFileFixtures


class KTUCloudNodeDriverTest(unittest.TestCase, TestCaseMixin):

    def setUp(self):
        KTUCloudNodeDriver.connectionCls.conn_class = KTUCloudStackMockHttp
        self.driver = KTUCloudNodeDriver('apikey', 'secret',
                                         path='/test/path',
                                         host='api.dummy.com')
        self.driver.path = '/test/path'
        self.driver.type = -1
        KTUCloudStackMockHttp.fixture_tag = 'default'
        self.driver.connection.poll_interval = 0.0

    def test_create_node_immediate_failure(self):
        size = self.driver.list_sizes()[0]
        image = self.driver.list_images()[0]
        KTUCloudStackMockHttp.fixture_tag = 'deployfail'
        self.assertRaises(
            Exception,
            self.driver.create_node,
            name='node-name', image=image, size=size)

    def test_create_node_delayed_failure(self):
        size = self.driver.list_sizes()[0]
        image = self.driver.list_images()[0]
        KTUCloudStackMockHttp.fixture_tag = 'deployfail2'
        self.assertRaises(
            Exception,
            self.driver.create_node,
            name='node-name', image=image, size=size)

    def test_list_images_no_images_available(self):
        KTUCloudStackMockHttp.fixture_tag = 'notemplates'

        images = self.driver.list_images()
        self.assertEqual(0, len(images))

    def test_list_images_available(self):
        images = self.driver.list_images()
        self.assertEqual(112, len(images))

    def test_list_sizes_available(self):
        sizes = self.driver.list_sizes()
        self.assertEqual(112, len(sizes))

    def test_list_sizes_nodisk(self):
        KTUCloudStackMockHttp.fixture_tag = 'nodisk'

        sizes = self.driver.list_sizes()
        self.assertEqual(2, len(sizes))

        check = False
        size = sizes[1]
        if size.id == KTUCloudNodeDriver.EMPTY_DISKOFFERINGID:
            check = True

        self.assertTrue(check)


class KTUCloudStackMockHttp(MockHttp, unittest.TestCase):
    fixtures = ComputeFileFixtures('ktucloud')
    fixture_tag = 'default'

    def _load_fixture(self, fixture):
        body = self.fixtures.load(fixture)
        return body, json.loads(body)

    def _test_path(self, method, url, body, headers):
        url = urlparse.urlparse(url)
        query = dict(parse_qsl(url.query))

        self.assertTrue('apiKey' in query)
        self.assertTrue('command' in query)
        self.assertTrue('response' in query)
        self.assertTrue('signature' in query)

        self.assertTrue(query['response'] == 'json')

        del query['apiKey']
        del query['response']
        del query['signature']
        command = query.pop('command')

        if hasattr(self, '_cmd_' + command):
            return getattr(self, '_cmd_' + command)(**query)
        else:
            fixture = command + '_' + self.fixture_tag + '.json'
            body, obj = self._load_fixture(fixture)
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _cmd_queryAsyncJobResult(self, jobid):
        fixture = 'queryAsyncJobResult' + '_' + str(jobid) + '.json'
        body, obj = self._load_fixture(fixture)
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

if __name__ == '__main__':
    sys.exit(unittest.main())



#!/usr/bin/env python
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import os
import time

from libcloud.loadbalancer.base import Member, Algorithm
from libcloud.loadbalancer.types import Provider, State
from libcloud.loadbalancer.providers import get_driver


def main():
    cls = get_driver(Provider.RACKSPACE)
    driver = cls('username', 'api key', region='ord')

    balancers = driver.list_balancers()

    print(balancers)

    # creating a balancer which balances traffic across two
    # nodes: 192.168.86.1:80 and 192.168.86.2:8080. Balancer
    # itself listens on port 80/tcp
    new_balancer_name = 'testlb' + os.urandom(4).encode('hex')
    members = (Member(None, '192.168.86.1', 80),
               Member(None, '192.168.86.2', 8080))
    new_balancer = driver.create_balancer(name=new_balancer_name,
                                          algorithm=Algorithm.ROUND_ROBIN,
                                          port=80,
                                          protocol='http',
                                          members=members)

    print(new_balancer)

    # wait for balancer to become ready
    # NOTE: in real life code add timeout to not end up in
    # endless loop when things go wrong on provider side
    while True:
        balancer = driver.get_balancer(balancer_id=new_balancer.id)

        if balancer.state == State.RUNNING:
            break

        print('sleeping for 30 seconds for balancers to become ready')
        time.sleep(30)

    # fetch list of members
    members = balancer.list_members()
    print(members)

    # remove first member
    balancer.detach_member(members[0])

    # remove the balancer
    driver.destroy_balancer(new_balancer)

if __name__ == '__main__':
    main()



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import sys

from libcloud.compute.base import NodeImage, NodeAuthPassword, NodeLocation
from libcloud.utils.py3 import httplib
from libcloud.test import unittest, MockHttp
from libcloud.test.file_fixtures import ComputeFileFixtures
from libcloud.compute.types import Provider, NodeState
from libcloud.test.secrets import ONEANDONE_PARAMS
from libcloud.compute.providers import get_driver


class OneAndOneTests(unittest.TestCase):
    def setUp(self):
        oneAndOne = get_driver(Provider.ONEANDONE)
        oneAndOne.connectionCls.conn_class = OneAndOneMockHttp
        self.driver = oneAndOne(ONEANDONE_PARAMS)

    '''
    Function tests for listing items
    '''

    def test_list_sizes(self):
        sizes = self.driver.list_sizes()
        self.assertEqual(len(sizes), 4)

    def test_list_locations(self):
        sizes = self.driver.list_locations()
        self.assertEqual(len(sizes), 4)

    def test_list_images(self):
        appliances = self.driver.list_images("IMAGE")
        self.assertEqual(len(appliances), 102)

    def test_get_image(self):
        appliance = self.driver.get_image('img_1')
        self.assertNotEqual(appliance, None)

    def test_list_nodes(self):
        nodes = self.driver.list_nodes()
        self.assertEqual(len(nodes), 5)

        counter = 0

        for node in nodes:
            if counter == 0:
                self.assertEqual(node.id, '8A7D5122BDC173B6E52223878CEF2748')
                self.assertEqual(node.name, 'Docs Content Ubuntu 16.04-1')
                self.assertEqual(node.state, NodeState.RUNNING)
                self.assertEqual(node.public_ips, ['50.21.182.126'])
                self.assertEqual(node.private_ips, [])
            if counter == 1:
                self.assertEqual(node.id, 'E7D36EC025C73796035BF4F171379025')
                self.assertEqual(node.name,
                                 'Docs Content Test Server: CentOS 7-1')
                self.assertEqual(node.state, NodeState.RUNNING)
                self.assertEqual(node.public_ips, ['62.151.179.99'])
                self.assertEqual(node.private_ips, [])
            if counter == 2:
                self.assertEqual(node.id, 'DDDC4CCA34AAB08132FA1E40F9FEAC25')
                self.assertEqual(node.name, 'App Dev Server 5')
                self.assertEqual(node.state, NodeState.RUNNING)
                self.assertEqual(node.public_ips, ['70.35.206.196'])
                self.assertEqual(node.private_ips, [])
            if counter == 3:
                self.assertEqual(node.id, 'D5C5C1D01249DE9B88BE3DAE973AA090')
                self.assertEqual(node.name, 'Docs Test Server: CentOS 7-2')
                self.assertEqual(node.state, NodeState.RUNNING)
                self.assertEqual(node.public_ips, ['74.208.88.88'])
                self.assertEqual(node.private_ips, [])
            if counter == 4:
                self.assertEqual(node.id, 'FB1765588A90364835782061CE48BA8E')
                self.assertEqual(node.name,
                                 'Docs Content Test Server Ubuntu 16.04-2')
                self.assertEqual(node.state, NodeState.RUNNING)
                self.assertEqual(node.public_ips, ['70.35.206.233'])
                self.assertEqual(node.private_ips, [])
            counter += 1

    def test_create_node(self):
        node = self.driver.create_node(name="name",
                                       image=NodeImage(
                                           id="image_id",
                                           name=None,
                                           driver=self.driver),
                                       ex_fixed_instance_size_id="instance_id",
                                       location=NodeLocation(
                                           "datacenter_id",
                                           name="name",
                                           country="GB",
                                           driver=self.driver),
                                       auth=NodeAuthPassword("password"),
                                       ex_ip="1.1.1.1",
                                       ex_monitoring_policy_id="mp_id",
                                       ex_firewall_policy_id="fw_id",
                                       ex_loadbalancer_id="lb_id",
                                       ex_description="description",
                                       ex_power_on="SHUTDOWN")

        self.assertEqual(node.id, "E7D36EC025C73796035BF4F171379025")
        self.assertEqual(node.name, "Docs Content Test Server: CentOS 7-1")
        self.assertEqual(node.extra["description"], "My server description")

        self.assertIsNone(node.extra["status"]["percent"])
        self.assertEqual(node.extra["status"]["state"], "POWERED_ON")

        self.assertEqual(node.extra["image"]["id"],
                         "B5F778B85C041347BCDCFC3172AB3F3C")
        self.assertEqual(node.extra["image"]["name"], "centos7-64std")

        self.assertEqual(node.extra["datacenter"]["id"],
                         "908DC2072407C94C8054610AD5A53B8C")
        self.assertEqual(node.extra["datacenter"]["country_code"], "US")
        self.assertEqual(node.extra["datacenter"]["location"],
                         "United States of America")

        self.assertEqual(node.extra["hardware"]["fixed_instance_size_id"],
                         "65929629F35BBFBA63022008F773F3EB")
        self.assertEqual(node.extra["hardware"]["vcore"], 1)
        self.assertEqual(node.extra["hardware"]["hdds"][0]["id"],
                         "CDB278D95A92CB4C379A9CAAD6759F02")
        self.assertEqual(node.extra["hardware"]["hdds"][0]["size"], 40)
        self.assertEqual(node.extra["hardware"]["hdds"][0]["is_main"], True)
        self.assertEqual(node.extra["hardware"]["cores_per_processor"], 1)
        self.assertEqual(node.extra["hardware"]["vcore"], 1)
        self.assertEqual(node.extra["hardware"]["ram"], 1)

        self.assertEqual(node.extra["ips"][0]["id"],
                         "FDBE99EDD57F8596CBF71B6B64BD0A92")
        self.assertEqual(node.extra["ips"][0]["ip"], "62.151.179.99")

        self.assertIsNone(node.extra["monitoring_policy"])
        self.assertEqual(node.extra["alerts"], [])
        self.assertIsNone(node.extra["snapshot"])
        self.assertIsNone(node.extra["dvd"])
        self.assertIsNone(node.extra["private_networks"])

    def test_ex_list_datacenters(self):
        datacenters = self.driver.ex_list_datacenters()

        self.assertEqual(len(datacenters), 4)

    def test_ex_shutdown_server(self):
        server = self.driver.ex_shutdown_server("srv_1")
        self.assertNotEqual(server, None)

    def test_reboot_node(self):
        node = self.driver.ex_get_server("srv_1")
        node = self.driver.reboot_node(node)
        self.assertNotEqual(node, None)

    def test_ex_get_server(self):
        server = self.driver.ex_get_server("srv_1")
        self.assertNotEqual(server, None)

    def test_destroy_node(self):
        server = self.driver.ex_get_server("srv_1")

        node = self.driver.destroy_node(server)
        self.assertNotEqual(node, None)

    def test_get_server_hardware(self):
        hardware = self.driver.ex_get_server_hardware("srv_1")
        self.assertNotEqual(hardware, None)
        self.assertEqual(hardware['vcore'], 1)
        self.assertEqual(hardware['cores_per_processor'], 1)
        self.assertEqual(hardware['ram'], 2)
        self.assertEqual(hardware['fixed_instance_size_id'], 0)
        self.assertNotEqual(hardware['hdds'], None)
        self.assertEqual(hardware['hdds'][0]['id'],
                         '8C626C1A7005D0D1F527143C413D461E')
        self.assertEqual(hardware['hdds'][0]['is_main'], True)
        self.assertEqual(hardware['hdds'][0]['size'], 40)

    def test_rename_server(self):
        server = self.driver.ex_rename_server("srv_1", "name")
        self.assertNotEqual(server, None)

    def test_ex_modify_server_hardware(self):
        node = self.driver.ex_modify_server_hardware("srv_1", vcore=1)
        self.assertNotEqual(node, None)

    def test_add_hdd(self):
        node = self.driver.ex_add_hdd("srv_1", 1, True)
        self.assertNotEqual(node, None)

    def test_modify_hdd(self):
        node = self.driver.ex_modify_server_hardware("srv_1", "hdd_id", 50)
        self.assertNotEqual(node, None)

    def test_remove_hdd(self):
        node = self.driver.ex_remove_hdd("srv_1", "hdd_id")
        self.assertNotEqual(node, None)

    def test_ex_get_server_image(self):
        image = self.driver.ex_get_server_image("srv_1")
        self.assertNotEqual(image, None)
        self.assertEqual(image['id'], "76EBF29C1250167C8754B2B3D1C05F68")
        self.assertEqual(image['name'], "centos7-64std")

    def test_ex_reinstall_server_image(self):
        node = self.driver.ex_reinstall_server_image("srv_1", "img_id",
                                                     "password")
        self.assertNotEqual(node, None)

    def test_ex_list_server_ips(self):
        ips = self.driver.ex_list_server_ips("srv_1")
        self.assertEqual(len(ips), 1)

    def test_ex_get_server_ip(self):
        ip = self.driver.ex_get_server_ip("srv_1", "ip_id")
        self.assertNotEqual(ip, None)

    def test_ex_assign_server(self):
        node = self.driver.ex_assign_server_ip("srv_1", "IPV$")
        self.assertNotEqual(node, None)

    def test_ex_remove_server_ip(self):
        node = self.driver.ex_remove_server_ip("srv_1", "ip_id", keep_ip=True)
        self.assertNotEqual(node, None)

    def test_ex_create_firewall_policy(self):
        rules = [
            {
                "protocol": "TCP",
                "port_from": 80,
                "port_to": 80,
                "source": "0.0.0.0"
            },
            {
                "protocol": "TCP",
                "port_from": 443,
                "port_to": 443,
                "source": "0.0.0.0"
            }
        ]
        firewall = self.driver \
            .ex_create_firewall_policy("name", rules,
                                       description="desc")

        self.assertNotEqual(firewall, None)

    def test_ex_list_firewall_policies(self):
        firewall = self.driver.ex_list_firewall_policies()
        self.assertNotEqual(firewall, None)
        self.assertEqual(len(firewall), 2)

    def test_ex_get_firewall_policy(self):
        firewall = self.driver.ex_get_firewall_policy("fw_id")
        self.assertNotEqual(firewall, None)

    def test_ex_delete_firewall_policy(self):
        firewall = self.driver.ex_delete_firewall_policy("fw_id")
        self.assertNotEqual(firewall, None)

    def test_ex_get_server_firewall_policies(self):
        firewall = self.driver \
            .ex_get_server_firewall_policies("srv_id", "ip_id")
        self.assertNotEqual(firewall, None)

    def test_ex_add_server_firewall_policy(self):
        node = self.driver \
            .ex_add_server_firewall_policy("srv_id", "ip_id", "fw_id")
        self.assertNotEqual(node, None)

    def test_ex_list_shared_storages(self):
        storages = self.driver.ex_list_shared_storages()
        self.assertEqual(len(storages), 3)

    def test_ex_get_shared_storage(self):
        storage = self.driver.ex_get_shared_storage('storage_1')

        self.assertNotEqual(storage, None)
        self.assertEqual(storage['id'], "6AD2F180B7B666539EF75A02FE227084")
        self.assertEqual(storage['size'], 200)
        self.assertEqual(storage['state'], 'ACTIVE')
        self.assertEqual(storage['description'],
                         'My shared storage test description')
        self.assertEqual(storage['datacenter']['id'],
                         'D0F6D8C8ED29D3036F94C27BBB7BAD36')
        self.assertEqual(storage['datacenter']['location'], 'USA')
        self.assertEqual(storage['datacenter']['country_code'], 'US')
        self.assertEqual(storage['cloudpanel_id'], 'vid35780')
        self.assertEqual(storage['size_used'], '0.00')
        self.assertEqual(storage["cifs_path"], "vid50995.nas1.lanvid50995")
        self.assertEqual(storage["nfs_path"], "vid50995.nas1.lan/:vid50995")
        self.assertEqual(storage["name"], "My shared storage test")
        self.assertEqual(storage["creation_date"], "2015-05-06T08:33:25+00:00")
        self.assertEqual(storage['servers'][0]['id'],
                         '638ED28205B1AFD7ADEF569C725DD85F')
        self.assertEqual(storage['servers'][0]["name"], "My server 1")
        self.assertEqual(storage['servers'][0]["rights"], "RW")

    def test_ex_create_shared_storage(self):
        storage = self.driver.ex_create_shared_storage(
            name='TEST', size=2, datacenter_id='dc_id')
        self.assertNotEqual(storage, None)

    def test_ex_delete_shared_storage(self):
        storage = self.driver.ex_delete_shared_storage('storage_1')
        self.assertNotEqual(storage, None)

    def test_ex_attach_server_to_shared_storage(self):
        storage = self.driver.ex_attach_server_to_shared_storage(
            'storage_1', 'srv_1', 'RW')
        self.assertNotEqual(storage, None)

    def test_ex_get_shared_storage_server(self):
        storage = self.driver.ex_get_shared_storage_server(
            'storage_1', 'srv_1')
        self.assertNotEqual(storage, None)

    def test_ex_detach_server_from_shared_storage(self):
        storage = self.driver.ex_detach_server_from_shared_storage(
            'storage_1', 'srv_1')
        self.assertNotEqual(storage, None)

    def test_ex_create_load_balancers(self):
        rules = [
            {
                "protocol": "TCP",
                "port_balancer": 80,
                "port_server": 80,
                "source": "0.0.0.0"
            },
            {
                "protocol": "TCP",
                "port_balancer": 9999,
                "port_server": 8888,
                "source": "0.0.0.0"
            }
        ]
        load_balancer = self.driver. \
            ex_create_load_balancer(name='name',
                                    method='ROUNDROBIN',
                                    rules=rules,
                                    persistence=True,
                                    persistence_time=1)

        self.assertNotEqual(load_balancer, None)

    def test_ex_list_load_balancers(self):
        load_balancers = self.driver.ex_list_load_balancers()
        self.assertEqual(len(load_balancers), 2)

    def test_update_load_balancer(self):
        load_balancer = self.driver. \
            ex_update_load_balancer("lb_1", name='new name')
        self.assertNotEqual(load_balancer, None)

    def test_ex_add_servers_to_load_balancer(self):
        load_balancer = self.driver. \
            ex_add_servers_to_load_balancer('lb_1', server_ips=["1.1.1.1"])
        self.assertNotEqual(load_balancer, None)

    def test_ex_remove_server_from_load_balancer(self):
        load_balancer = self.driver. \
            ex_remove_server_from_load_balancer('lb_1', server_ip="srv_1")
        self.assertNotEqual(load_balancer, None)

    def test_ex_add_load_balancer_rule(self):
        load_balancer = self.driver. \
            ex_add_load_balancer_rule('lb_1', protocol='TCP', port_balancer=82,
                                      port_server=81, source='0.0.0.0')
        self.assertNotEqual(load_balancer, None)

    def test_ex_remove_load_balancer_rule(self):
        load_balancer = self.driver. \
            ex_remove_load_balancer_rule('lb_1', 'rule_1')
        self.assertNotEqual(load_balancer, None)

    def test_ex_get_load_balancer(self):
        load_balancer = self.driver. \
            ex_get_load_balancer('lb_1')
        self.assertNotEqual(load_balancer, None)

    def test_ex_get_load_balancer_server_ip(self):
        server_ip = self.driver. \
            ex_get_load_balancer_server_ip('lb_1', 'srv_1')
        self.assertNotEqual(server_ip, None)

    def test_ex_list_load_balancer_rules(self):
        rules = self.driver. \
            ex_list_load_balancer_rules('lb_1')
        self.assertNotEqual(rules, None)
        self.assertEqual(len(rules), 2)

    def test_ex_get_load_balancer_rule(self):
        rule = self.driver. \
            ex_get_load_balancer_rule('lb_1', 'rule_1')
        self.assertNotEqual(rule, None)

    def test_ex_delete_load_balancer(self):
        load_balancer = self.driver. \
            ex_delete_load_balancer('lb_1')
        self.assertNotEqual(load_balancer, None)

    def test_ex_list_public_ips(self):
        ips = self.driver.ex_list_public_ips()
        self.assertNotEqual(ips, None)
        self.assertEqual(len(ips), 3)

    def test_ex_create_public_ip(self):
        ip = self.driver.ex_create_public_ip('IPv4')
        self.assertNotEqual(ip, None)

    def test_ex_get_public_ip(self):
        ip = self.driver.ex_get_public_ip('ip_1')
        self.assertNotEqual(ip, None)

    def test_ex_delete_public_ip(self):
        ip = self.driver.ex_delete_public_ip('ip_1')
        self.assertNotEqual(ip, None)

    def test_ex_update_public_ip(self):
        ip = self.driver.ex_update_public_ip('ip_1', "reverse.dns")
        self.assertNotEqual(ip, None)

    def test_ex_create_monitoring_policy(self):
        thresholds = {
            "cpu": {
                "warning": {
                    "value": 90,
                    "alert": False
                },
                "critical": {
                    "value": 95,
                    "alert": False
                }
            },
            "ram": {
                "warning": {
                    "value": 90,
                    "alert": False
                },
                "critical": {
                    "value": 95,
                    "alert": False
                }
            },
            "disk": {
                "warning": {
                    "value": 80,
                    "alert": False
                },
                "critical": {
                    "value": 90,
                    "alert": False
                }
            },
            "transfer": {
                "warning": {
                    "value": 1000,
                    "alert": False
                },
                "critical": {
                    "value": 2000,
                    "alert": False
                }
            },
            "internal_ping": {
                "warning": {
                    "value": 50,
                    "alert": False
                },
                "critical": {
                    "value": 100,
                    "alert": False
                }
            }
        }

        ports = [
            {
                "protocol": "TCP",
                "port": "22",
                "alert_if": "RESPONDING",
                "email_notification": True
            }
        ]

        processes = [
            {
                "process": "test",
                "alert_if": "NOT_RUNNING",
                "email_notification": True
            }
        ]

        policy = self.driver. \
            ex_create_monitoring_policy(name='test_name',
                                        thresholds=thresholds,
                                        ports=ports,
                                        processes=processes,
                                        description='description',
                                        email='test@domain.com',
                                        agent=True)
        self.assertNotEqual(policy, None)

    def test_ex_list_monitoring_policies(self):
        policies = self.driver.ex_list_monitoring_policies()
        self.assertNotEqual(policies, None)
        self.assertEqual(len(policies), 2)

    def test_ex_get_monitoring_policy(self):
        policy = self.driver.ex_get_monitoring_policy('pol_1')
        self.assertNotEqual(policy, None)

    def test_ex_update_monitoring_policy(self):
        thresholds = {
            "cpu": {
                "warning": {
                    "value": 90,
                    "alert": False
                },
                "critical": {
                    "value": 95,
                    "alert": False
                }
            },
            "ram": {
                "warning": {
                    "value": 90,
                    "alert": False
                },
                "critical": {
                    "value": 95,
                    "alert": False
                }
            },
            "disk": {
                "warning": {
                    "value": 80,
                    "alert": False
                },
                "critical": {
                    "value": 90,
                    "alert": False
                }
            },
            "transfer": {
                "warning": {
                    "value": 1000,
                    "alert": False
                },
                "critical": {
                    "value": 2000,
                    "alert": False
                }
            },
            "internal_ping": {
                "warning": {
                    "value": 50,
                    "alert": False
                },
                "critical": {
                    "value": 100,
                    "alert": False
                }
            }
        }

        policy = self.driver. \
            ex_update_monitoring_policy('pol_1', email='test@domain.com',
                                        thresholds=thresholds,
                                        name='new name',
                                        description='new description')
        self.assertNotEqual(policy, None)

    def test_ex_get_monitoring_policy_ports(self):
        ports = self.driver. \
            ex_get_monitoring_policy_ports('pol_1')
        self.assertNotEqual(ports, None)
        self.assertEqual(len(ports), 2)

    def test_ex_get_monitoring_policy_port(self):
        port = self.driver. \
            ex_get_monitoring_policy_port('pol_1', 'port_1')
        self.assertNotEqual(port, None)

    def test_ex_remove_monitoring_policy_port(self):
        port = self.driver. \
            ex_remove_monitoring_policy_port('pol_1', 'port_1')
        self.assertNotEqual(port, None)

    def test_ex_add_monitoring_policy_ports(self):
        new_ports = [
            {
                "protocol": "TCP",
                "port": "80",
                "alert_if": "RESPONDING",
                "email_notification": True
            }
        ]
        ports = self.driver. \
            ex_add_monitoring_policy_ports('pol_1', new_ports)
        self.assertNotEqual(ports, None)
        self.assertEqual(len(ports), 2)

    def test_ex_get_monitoring_policy_processes(self):
        processes = self.driver. \
            ex_get_monitoring_policy_processes('pol_1')
        self.assertNotEqual(processes, None)

    def test_ex_get_monitoring_policy_process(self):
        process = self.driver. \
            ex_get_monitoring_policy_process('pol_1', 'proc_1')
        self.assertNotEqual(process, None)

    def test_ex_remove_monitoring_policy_process(self):
        policy = self.driver. \
            ex_remove_monitoring_policy_process('pol_1', 'proc_1')
        self.assertNotEqual(policy, None)

    def test_ex_add_monitoring_policy_processes(self):
        processes = {
            "processes": [
                {
                    "process": "taskmmgr",
                    "alert_if": "RUNNING",
                    "email_notification": True
                }
            ]
        }
        processes = self.driver. \
            ex_add_monitoring_policy_processes(policy_id='pol_1',
                                               processes=processes)
        self.assertNotEqual(processes, None)
        self.assertEqual(len(processes), 2)

    def test_ex_list_monitoring_policy_servers(self):
        servers = self.driver.ex_list_monitoring_policy_servers('pol_1')
        self.assertNotEqual(servers, None)
        self.assertEqual(len(servers), 2)

    def test_ex_add_servers_to_monitoring_policy(self):
        servers = self.driver. \
            ex_add_servers_to_monitoring_policy('pol_1', 'serv_1')
        self.assertNotEqual(servers, None)
        self.assertEqual(len(servers), 2)

    def test_ex_remove_server_from_monitoring_policy(self):
        policy = self.driver. \
            ex_remove_server_from_monitoring_policy('pol_1', 'serv_1')
        self.assertNotEqual(policy, None)


class OneAndOneMockHttp(MockHttp):
    fixtures = ComputeFileFixtures('oneandone')

    '''
    Operation on Server Appliances

    GET - Fetches Server Appliances
    '''

    def _v1_server_appliances(self, method, url, body, headers):
        body = self.fixtures.load('list_images.json')
        return (
            httplib.OK,
            body,
            {},
            httplib.responses[httplib.OK]
        )

    def _v1_server_appliances_img_1(self, method, url, body, headers):
        body = self.fixtures.load('get_image.json')
        return (
            httplib.OK,
            body,
            {},
            httplib.responses[httplib.OK]
        )

    def _v1_servers(self, method, url, body, headers):
        if method == "GET":
            body = self.fixtures.load('list_servers.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )
        if method == "POST":
            body = self.fixtures.load('create_node.json')
            return (
                httplib.ACCEPTED,
                body,
                {},
                httplib.responses[httplib.ACCEPTED]
            )

    def _v1_create_node(self, method, url, body_headers):

        body = self.fixtures.load('list_servers.json')

        return (
            httplib.ACCEPTED,
            {},
            body,
            httplib.responses[httplib.ACCEPTED]
        )

    def _v1_datacenters(
        self, method, url, body, headers
    ):
        if method == 'GET':
            body = self.fixtures.load('ex_list_datacenters.json')
            return (
                httplib.OK,
                body,
                {'content-type': 'application/json'},
                httplib.responses[httplib.OK]
            )

    def _v1_servers_srv_1(
        self, method, url, body, headers
    ):
        pass

        if method == 'PUT':
            body = self.fixtures.load('describe_server.json')
            return (
                httplib.OK,
                body,
                {'content-type': 'application/json'},
                httplib.responses[httplib.OK]
            )
        if method == 'GET':
            body = self.fixtures.load('describe_server.json')
            return (
                httplib.OK,
                body,
                {'content-type': 'application/json'},
                httplib.responses[httplib.OK]
            )
        if method == 'DELETE':
            body = self.fixtures.load('describe_server.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )

    def _v1_servers_srv_1_status_action(self, method, url, body_headers, id):
        body = self.fixtures.load('describe_server.json')

        return (
            httplib.ACCEPTED,
            body,
            {},
            httplib.responses[httplib.ACCEPTED]
        )

    def _v1_servers_srv_1_hardware(
        self, method, url, body, headers
    ):
        if method == 'GET':
            body = self.fixtures.load('server_hardware.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )
        if method == 'PUT':
            body = self.fixtures.load('describe_server.json')
            return (
                httplib.ACCEPTED,
                body,
                {},
                httplib.responses[httplib.ACCEPTED]
            )

    def _v1_servers_srv_1_hardware_hdds(
        self, method, url, body, headers
    ):
        if method == 'POST':
            body = self.fixtures.load('describe_server.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )
        if method == 'PUT':
            body = self.fixtures.load('describe_server.json')
            return (
                httplib.ACCEPTED,
                body,
                {},
                httplib.responses[httplib.ACCEPTED]
            )

    def _v1_servers_srv_1_hardware_hdds_hdd_id(
        self, method, url, body, headers
    ):
        if method == 'DELETE':
            body = self.fixtures.load('describe_server.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )
        if method == 'PUT':
            body = self.fixtures.load('describe_server.json')
            return (
                httplib.ACCEPTED,
                body,
                {},
                httplib.responses[httplib.ACCEPTED]
            )

    def _v1_servers_srv_1_image(
        self, method, url, body, headers
    ):
        if method == 'GET':
            body = self.fixtures.load('get_server_image.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )
        if method == 'PUT':
            body = self.fixtures.load('describe_server.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )

    def _v1_servers_srv_1_ips(
        self, method, url, body, headers
    ):

        if method == 'GET':
            body = self.fixtures.load('server_ips.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )
        if method == 'POST':
            body = self.fixtures.load('describe_server.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )

    def _v1_servers_srv_1_ips_ip_id(
        self, method, url, body, headers
    ):
        if method == 'GET':
            body = self.fixtures.load('server_ip.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )
        if method == 'DELETE':
            body = self.fixtures.load('describe_server.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )

    def _v1_firewall_policies(
        self, method, url, body, headers
    ):

        if method == 'POST':
            body = self.fixtures.load('describe_firewall_policy.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )

        if method == 'GET':
            body = self.fixtures.load('list_firewall_policies.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )

    def _v1_firewall_policy_fw_id(
        self, method, url, body, headers
    ):
        if method == 'GET':
            body = self.fixtures.load('describe_firewall_policy.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )

        if method == 'DELETE':
            body = self.fixtures.load('describe_firewall_policy.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )

    def _v1_servers_srv_id_ips_ip_id_firewall_policy(
        self, method, url, body, header
    ):
        if method == 'GET':
            body = self.fixtures.load('describe_id_firewall_policy.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )
        if method == 'DELETE':
            body = self.fixtures.load('describe_server.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )
        if method == 'POST':
            body = self.fixtures.load('describe_server.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )

    def _v1_shared_storages(
        self, method, url, body, header
    ):
        if method == 'GET' or method == 'POST':
            body = self.fixtures.load('list_shared_storages.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )

    def _v1_shared_storages_storage_1(
        self, method, url, body, header
    ):
        if method == 'GET' or method == 'DELETE':
            body = self.fixtures.load('shared_storage.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )

    def _v1_shared_storages_storage_1_servers(
        self, method, url, body, header
    ):
        if method == 'POST' or method == 'DELETE':
            body = self.fixtures.load('shared_storage.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )

    def _v1_shared_storages_storage_1_servers_srv_1(
        self, method, url, body, header
    ):
        if method == 'GET' or method == 'DELETE':
            body = self.fixtures.load('shared_storage.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )

    def _v1_load_balancers(
        self, method, url, body, header
    ):
        if method == 'POST':
            body = self.fixtures.load('load_balancer.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )
        if method == 'GET':
            body = self.fixtures.load('list_load_balancer.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )

    def _v1_load_balancers_lb_1(
        self, method, url, body, header
    ):

        body = self.fixtures.load('load_balancer.json')
        return (
            httplib.OK,
            body,
            {},
            httplib.responses[httplib.OK]
        )

    def _v1_load_balancers_lb_1_server_ips(
        self, method, url, body, header
    ):
        if method == 'GET':
            body = self.fixtures.load('load_balancer_server_ips.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )

        if method == 'POST':
            body = self.fixtures.load('load_balancer.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )

    def _v1_load_balancers_lb_1_rules(
        self, method, url, body, header
    ):
        if method == 'POST':
            body = self.fixtures.load('load_balancer.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )
        if method == 'GET':
            body = self.fixtures.load('load_balancer_rules.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )

    def _v1_load_balancers_lb_1_server_ips_srv_1(
        self, method, url, body, header
    ):
        if method == 'DELETE':
            body = self.fixtures.load('load_balancer.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )

        if method == 'GET':
            body = self.fixtures.load('load_balancer_server_ip.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )

    def _v1_load_balancers_lb_1_rules_rule_1(
        self, method, url, body, header
    ):
        if method == 'DELETE':
            body = self.fixtures.load('load_balancer.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )
        if method == 'GET':
            body = self.fixtures.load('load_balancer_rule.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )

    def _v1_public_ips(
        self, method, url, body, header
    ):
        if method == 'GET':
            body = self.fixtures.load('list_public_ips.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )
        if method == 'POST':
            body = self.fixtures.load('public_ip.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )

    def _v1_public_ips_ip_1(
        self, method, url, body, header
    ):
        body = self.fixtures.load('public_ip.json')
        return (
            httplib.OK,
            body,
            {},
            httplib.responses[httplib.OK]
        )

    def _v1_monitoring_policies(
        self, method, url, body, header
    ):
        if method == 'POST':
            body = self.fixtures.load('monitoring_policy.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )
        if method == 'GET':
            body = self.fixtures.load('list_monitoring_policies.json')
            return (
                httplib.OK,
                body,
                {},
                httplib.responses[httplib.OK]
            )

    def _v1_monitoring_policies_pol_1(
        self, method, url, body, header
    ):
        body = self.fixtures.load('monitoring_policy.json')
        return (
            httplib.OK,
            body,
            {},
            httplib.responses[httplib.OK]
        )

    def _v1_monitoring_policies_pol_1_ports(
        self, method, url, body, header
    ):
        body = self.fixtures.load('monitoring_policy_ports.json')
        return (
            httplib.OK,
            body,
            {},
            httplib.responses[httplib.OK]
        )

    def _v1_monitoring_policies_pol_1_ports_port_1(
        self, method, url, body, header
    ):
        if method == 'GET':
            body = self.fixtures.load('monitoring_policy_port.json')
        elif method == 'POST':
            body = self.fixtures.load('monitoring_policy.json')

        return (
            httplib.OK,
            body,
            {},
            httplib.responses[httplib.OK]
        )

    def _v1_monitoring_policies_pol_1_processes(
        self, method, url, body, header
    ):
        body = self.fixtures.load('monitoring_policy_processes.json')
        return (
            httplib.OK,
            body,
            {},
            httplib.responses[httplib.OK]
        )

    def _v1_monitoring_policies_pol_1_processes_proc_1(
        self, method, url, body, header
    ):
        if method == 'GET':
            body = self.fixtures.load('monitoring_policy_process.json')
        elif method == 'POST':
            body = self.fixtures.load('monitoring_policy.json')

        return (
            httplib.OK,
            body,
            {},
            httplib.responses[httplib.OK]
        )

    def _v1_monitoring_policies_pol_1_servers(
        self, method, url, body, header
    ):
        body = self.fixtures.load('monitoring_policy_servers.json')
        return (
            httplib.OK,
            body,
            {},
            httplib.responses[httplib.OK]
        )

    def _v1_monitoring_policies_pol_1_servers_serv_1(
        self, method, url, body, header
    ):
        if method == 'GET':
            body = self.fixtures.load('monitoring_policy_servers.json')
        elif method == 'POST':
            body = self.fixtures.load('monitoring_policy.json')

        return (
            httplib.OK,
            body,
            {},
            httplib.responses[httplib.OK]
        )

    def _v1_servers_fixed_instance_sizes(
        self, method, url, body, header
    ):
        body = self.fixtures.load('fixed_instance_sizes.json')
        return (
            httplib.OK,
            body,
            {},
            httplib.responses[httplib.OK]
        )


if __name__ == '__main__':
    sys.exit(unittest.main())



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys

from libcloud.compute.drivers.vsphere import VSphere_REST_NodeDriver

from libcloud.utils.py3 import httplib

from libcloud.test import unittest
from libcloud.test import MockHttp
from libcloud.test.file_fixtures import ComputeFileFixtures


class KubeVirtTestCase(unittest.TestCase):

    driver_cls = VSphere_REST_NodeDriver
    fixtures = ComputeFileFixtures('vsphere')

    def setUp(self):
        VSphere_REST_NodeDriver.connectionCls.conn_class = VSphereMockHttp
        self.driver = VSphere_REST_NodeDriver(key='user',
                                              secret='pass',
                                              secure=True,
                                              host='foo',
                                              port=443)

    def test_list_nodes(self):
        vm_id = "vm-80"
        nodes = self.driver.list_nodes()
        self.assertEqual(len(nodes), 1)
        self.assertEqual(nodes[0].name, 'vCenter')
        self.assertEqual(nodes[0].id, vm_id)

    def test_list_locations(self):
        locations = self.driver.list_locations()
        self.assertEqual(len(locations), 1)
        self.assertEqual(locations[0].id, "host-74")
        self.assertEqual(locations[0].name, "100.100.100.100")

    def test_destroy_node(self):
        nodes = self.driver.list_nodes()
        to_destroy = nodes[-1]
        resp = self.driver.destroy_node(to_destroy)
        self.assertTrue(resp)

    def test_start_node(self):
        nodes = self.driver.list_nodes()
        resp = self.driver.start_node(nodes[0])
        self.assertTrue(resp)

    def test_stop_node(self):
        nodes = self.driver.list_nodes()
        resp = self.driver.stop_node(nodes[0])
        self.assertTrue(resp)

    def test_reboot_node(self):
        nodes = self.driver.list_nodes()
        resp = self.driver.reboot_node(nodes[0])
        self.assertTrue(resp)


class VSphereMockHttp(MockHttp):

    fixtures = ComputeFileFixtures('vsphere')

    def _rest_com_vmware_cis_session(self, method, url, body, headers):
        if method == "POST":
            body = self.fixtures.load('session_token.json')
        else:
            raise AssertionError('Unsupported method')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _rest_vcenter_vm(self, method, url, body, headers):
        if method == "GET":
            body = self.fixtures.load('list_nodes.json')
        elif method == "POST":
            return
        else:
            raise AssertionError('Unsupported method')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _rest_vcenter_vm_vm_80(self, method, url, body, headers):
        if method == "GET":
            body = self.fixtures.load('node_80.json')
        elif method == "POST":
            return
        elif method == "DELETE":
            body = ""
        else:
            raise AssertionError('Unsupported method')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _rest_vcenter_cluster(self, method, url, body, headers):
        if method == "GET":
            body = self.fixtures.load('list_clusters.json')
        else:
            raise AssertionError('Unsupported method')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _rest_vcenter_host(self, method, url, body, headers):
        if method == "GET":
            body = self.fixtures.load('list_hosts.json')
        else:
            raise AssertionError('Unsupported method')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _rest_appliance_networking_interfaces(self, method, url, body, headers):
        if method == "GET":
            body = self.fixtures.load('list_interfaces.json')
        else:
            raise AssertionError('Unsupported method')
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _rest_vcenter_vm_vm_80_power_stop(self, method, url, body, headers):
        if method != "POST":
            raise AssertionError('Unsupported method')
        body = ""

        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _rest_vcenter_vm_vm_80_power_start(self, method, url, body, headers):
        if method != "POST":
            raise AssertionError('Unsupported method')
        body = ""

        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _rest_vcenter_vm_vm_80_power_reset(self, method, url, body, headers):
        if method != "POST":
            raise AssertionError('Unsupported method')
        body = ""

        return (httplib.OK, body, {}, httplib.responses[httplib.OK])


if __name__ == "__main__":
    sys.exit(unittest.main())



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from libcloud.common.types import LibcloudError
from libcloud.common.aws import SignedAWSConnection
from libcloud.storage.drivers.s3 import BaseS3Connection, S3Connection
from libcloud.storage.drivers.s3 import BaseS3StorageDriver

__all__ = [
    'DigitalOceanSpacesStorageDriver'
]

DO_SPACES_HOSTS_BY_REGION = {'nyc3': 'nyc3.digitaloceanspaces.com',
                             'ams3': 'ams3.digitaloceanspaces.com',
                             'sfo2': 'sfo2.digitaloceanspaces.com',
                             'sgp1': 'sgp1.digitaloceanspaces.com'}
DO_SPACES_DEFAULT_REGION = 'nyc3'
DEFAULT_SIGNATURE_VERSION = '2'
S3_API_VERSION = '2006-03-01'


class DOSpacesConnectionAWS4(SignedAWSConnection, BaseS3Connection):
    service_name = 's3'
    version = S3_API_VERSION

    def __init__(self, user_id, key, secure=True, host=None, port=None,
                 url=None, timeout=None, proxy_url=None, token=None,
                 retry_delay=None, backoff=None, **kwargs):

        super(DOSpacesConnectionAWS4, self).__init__(user_id, key,
                                                     secure, host,
                                                     port, url,
                                                     timeout,
                                                     proxy_url, token,
                                                     retry_delay,
                                                     backoff,
                                                     signature_version=4)


class DOSpacesConnectionAWS2(S3Connection):

    def __init__(self, user_id, key, secure=True, host=None, port=None,
                 url=None, timeout=None, proxy_url=None, token=None,
                 retry_delay=None, backoff=None, **kwargs):

        super(DOSpacesConnectionAWS2, self).__init__(user_id, key,
                                                     secure, host,
                                                     port, url,
                                                     timeout,
                                                     proxy_url, token,
                                                     retry_delay,
                                                     backoff)


class DigitalOceanSpacesStorageDriver(BaseS3StorageDriver):
    name = 'DigitalOcean Spaces'
    website = 'https://www.digitalocean.com/products/object-storage/'
    supports_chunked_encoding = False
    supports_s3_multipart_upload = True

    def __init__(self, key, secret=None, secure=True, host=None, port=None,
                 api_version=None, region=DO_SPACES_DEFAULT_REGION, **kwargs):

        if region not in DO_SPACES_HOSTS_BY_REGION:
            raise LibcloudError('Unknown region (%s)' % (region), driver=self)

        host = DO_SPACES_HOSTS_BY_REGION[region]
        self.name = 'DigitalOcean Spaces (%s)' % (region)

        self.region_name = region
        self.signature_version = str(kwargs.pop('signature_version',
                                                DEFAULT_SIGNATURE_VERSION))

        if self.signature_version == '2':
            self.connectionCls = DOSpacesConnectionAWS2
        elif self.signature_version == '4':
            self.connectionCls = DOSpacesConnectionAWS4
        else:
            raise ValueError('Invalid signature_version: %s' %
                             (self.signature_version))
        self.connectionCls.host = host

        super(DigitalOceanSpacesStorageDriver,
              self).__init__(key, secret,
                             secure, host, port,
                             api_version, region,
                             **kwargs)

    def _ex_connection_class_kwargs(self):
        kwargs = {}
        kwargs['signature_version'] = self.signature_version
        return kwargs



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Joyent Cloud (http://www.joyentcloud.com) driver.
"""

import base64

try:
    import simplejson as json
except Exception:
    import json

from libcloud.utils.py3 import httplib
from libcloud.utils.py3 import b

from libcloud.common.types import LibcloudError
from libcloud.compute.providers import Provider
from libcloud.common.base import JsonResponse, ConnectionUserAndKey
from libcloud.compute.types import NodeState, InvalidCredsError
from libcloud.compute.base import Node, NodeDriver, NodeImage, NodeSize
from libcloud.utils.networking import is_private_subnet

API_HOST_SUFFIX = '.api.joyentcloud.com'
API_VERSION = '~6.5'


NODE_STATE_MAP = {
    'provisioning': NodeState.PENDING,
    'running': NodeState.RUNNING,
    'stopping': NodeState.TERMINATED,
    'stopped': NodeState.TERMINATED,
    'deleted': NodeState.TERMINATED
}

VALID_REGIONS = [
    'us-east-1', 'us-east-2', 'us-east-3',
    'us-west-1',
    'us-sw-1',
    'eu-ams-1'
]
DEFAULT_REGION = 'us-east-1'


class JoyentResponse(JsonResponse):
    """
    Joyent response class.
    """

    valid_response_codes = [httplib.OK, httplib.ACCEPTED, httplib.CREATED,
                            httplib.NO_CONTENT]

    def parse_error(self):
        if self.status == httplib.UNAUTHORIZED:
            data = self.parse_body()
            raise InvalidCredsError(data['code'] + ': ' + data['message'])
        return self.body

    def success(self):
        return self.status in self.valid_response_codes


class JoyentConnection(ConnectionUserAndKey):
    """
    Joyent connection class.
    """

    responseCls = JoyentResponse

    allow_insecure = False

    def add_default_headers(self, headers):
        headers['Accept'] = 'application/json'
        headers['Content-Type'] = 'application/json; charset=UTF-8'
        headers['X-Api-Version'] = API_VERSION

        user_b64 = base64.b64encode(b('%s:%s' % (self.user_id, self.key)))
        headers['Authorization'] = 'Basic %s' % (user_b64.decode('utf-8'))
        return headers


class JoyentNodeDriver(NodeDriver):
    """
    Joyent node driver class.
    """

    type = Provider.JOYENT
    name = 'Joyent'
    website = 'http://www.joyentcloud.com'
    connectionCls = JoyentConnection
    features = {'create_node': ['generates_password']}

    def __init__(self, key, secret=None, secure=True, host=None, port=None,
                 region=DEFAULT_REGION, **kwargs):
        # Location is here for backward compatibility reasons
        if 'location' in kwargs:
            region = kwargs['location']

        if region not in VALID_REGIONS:
            msg = 'Invalid region: "%s". Valid region: %s'
            raise LibcloudError(msg % (region,
                                ', '.join(VALID_REGIONS)), driver=self)

        super(JoyentNodeDriver, self).__init__(key=key, secret=secret,
                                               secure=secure, host=host,
                                               port=port, region=region,
                                               **kwargs)
        self.connection.host = region + API_HOST_SUFFIX

    def list_images(self):
        result = self.connection.request('/my/datasets').object

        images = []
        for value in result:
            extra = {'type': value['type'], 'urn': value['urn'],
                     'os': value['os'], 'default': value['default']}
            image = NodeImage(id=value['id'], name=value['name'],
                              driver=self.connection.driver, extra=extra)
            images.append(image)

        return images

    def list_sizes(self):
        result = self.connection.request('/my/packages').object

        sizes = []
        for value in result:
            size = NodeSize(id=value['name'], name=value['name'],
                            ram=value['memory'], disk=value['disk'],
                            bandwidth=None, price=0.0,
                            driver=self.connection.driver)
            sizes.append(size)

        return sizes

    def list_nodes(self):
        result = self.connection.request('/my/machines').object

        nodes = []
        for value in result:
            node = self._to_node(value)
            nodes.append(node)

        return nodes

    def reboot_node(self, node):
        data = json.dumps({'action': 'reboot'})
        result = self.connection.request('/my/machines/%s' % (node.id),
                                         data=data, method='POST')
        return result.status == httplib.ACCEPTED

    def destroy_node(self, node):
        result = self.connection.request('/my/machines/%s' % (node.id),
                                         method='DELETE')
        return result.status == httplib.NO_CONTENT

    def create_node(self, name, size, image):
        data = json.dumps({'name': name, 'package': size.id,
                           'dataset': image.id})
        result = self.connection.request('/my/machines', data=data,
                                         method='POST')
        return self._to_node(result.object)

    def start_node(self, node):
        """
        Start node

        :param  node: The node to be stopped
        :type   node: :class:`Node`

        :rtype: ``bool``
        """
        data = json.dumps({'action': 'start'})
        result = self.connection.request('/my/machines/%s' % (node.id),
                                         data=data, method='POST')
        return result.status == httplib.ACCEPTED

    def stop_node(self, node):
        """
        Stop node

        :param  node: The node to be stopped
        :type   node: :class:`Node`

        :rtype: ``bool``
        """
        data = json.dumps({'action': 'stop'})
        result = self.connection.request('/my/machines/%s' % (node.id),
                                         data=data, method='POST')
        return result.status == httplib.ACCEPTED

    def ex_start_node(self, node):
        # NOTE: This method is here for backward compatibility reasons after
        # this method was promoted to be part of the standard compute API in
        # Libcloud v2.7.0
        return self.start_node(node=node)

    def ex_stop_node(self, node):
        # NOTE: This method is here for backward compatibility reasons after
        # this method was promoted to be part of the standard compute API in
        # Libcloud v2.7.0
        return self.stop_node(node=node)

    def ex_get_node(self, node_id):
        """
        Return a Node object based on a node ID.

        :param  node_id: ID of the node
        :type   node_id: ``str``

        :return:  A Node object for the node
        :rtype:   :class:`Node`
        """
        result = self.connection.request('/my/machines/%s' % (node_id))
        return self._to_node(result.object)

    def _to_node(self, data):
        state = NODE_STATE_MAP[data['state']]
        public_ips = []
        private_ips = []
        extra = {}

        for ip in data['ips']:
            if is_private_subnet(ip):
                private_ips.append(ip)
            else:
                public_ips.append(ip)

        if 'credentials' in data['metadata']:
            extra['password'] = data['metadata']['credentials']['root']

        node = Node(id=data['id'], name=data['name'], state=state,
                    public_ips=public_ips, private_ips=private_ips,
                    driver=self.connection.driver, extra=extra)
        return node



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import unittest

from libcloud.test import MockHttp
from libcloud.test.file_fixtures import DNSFileFixtures
from libcloud.test.secrets import DNS_PARAMS_NSONE
from libcloud.dns.drivers.nsone import NsOneDNSDriver
from libcloud.utils.py3 import httplib
from libcloud.common.nsone import NsOneException
from libcloud.dns.types import ZoneDoesNotExistError, ZoneAlreadyExistsError,\
    RecordDoesNotExistError, RecordType, RecordAlreadyExistsError
from libcloud.dns.base import Zone, Record

class NsOneTests(unittest.TestCase):
    def setUp(self):
        NsOneMockHttp.type = None
        NsOneDNSDriver.connectionCls.conn_class = NsOneMockHttp
        self.driver = NsOneDNSDriver(*DNS_PARAMS_NSONE)
        self.test_zone = Zone(id='test.com', type='master', ttl=None,
                              domain='test.com', extra={}, driver=self)
        self.test_record = Record(id='13', type=RecordType.A,
                                  name='example.com', zone=self.test_zone,
                                  data='127.0.0.1', driver=self, extra={})

    def test_list_zones_empty(self):
        NsOneMockHttp.type = 'EMPTY_ZONES_LIST'
        zones = self.driver.list_zones()

        self.assertEqual(zones, [])

    def test_list_zones_success(self):
        zones = self.driver.list_zones()

        self.assertEqual(len(zones), 2)

        zone = zones[0]
        self.assertEqual(zone.id, '520422af9f782d37dffb588b')
        self.assertIsNone(zone.type)
        self.assertEqual(zone.domain, 'example.com')
        self.assertEqual(zone.ttl, 3600)

        zone = zones[1]
        self.assertEqual(zone.id, '520422c99f782d37dffb5892')
        self.assertIsNone(zone.type)
        self.assertEqual(zone.domain, 'nsoneisgreat.com')
        self.assertEqual(zone.ttl, 3600)

    def test_delete_zone_zone_does_not_exist(self):
        NsOneMockHttp.type = 'DELETE_ZONE_ZONE_DOES_NOT_EXIST'

        try:
            self.driver.delete_zone(zone=self.test_zone)
        except ZoneDoesNotExistError as e:
            self.assertEqual(e.zone_id, self.test_zone.id)
        else:
            self.fail('Exception was not thrown')

    def test_delete_zone_success(self):
        NsOneMockHttp.type = 'DELETE_ZONE_SUCCESS'
        status = self.driver.delete_zone(zone=self.test_zone)

        self.assertTrue(status)

    def test_get_zone_zone_does_not_exist(self):
        NsOneMockHttp.type = 'GET_ZONE_ZONE_DOES_NOT_EXIST'
        try:
            self.driver.get_zone(zone_id='zonedoesnotexist.com')
        except ZoneDoesNotExistError as e:
            self.assertEqual(e.zone_id, 'zonedoesnotexist.com')
        else:
            self.fail('Exception was not thrown')

    def test_create_zone_success(self):
        NsOneMockHttp.type = 'CREATE_ZONE_SUCCESS'
        zone = self.driver.create_zone(domain='newzone.com')

        self.assertEqual(zone.id, '52051b2c9f782d58bb4df41b')
        self.assertEqual(zone.domain, 'newzone.com')
        self.assertIsNone(zone.type),
        self.assertEqual(zone.ttl, 3600)

    def test_create_zone_zone_already_exists(self):
        NsOneMockHttp.type = 'CREATE_ZONE_ZONE_ALREADY_EXISTS'

        try:
            self.driver.create_zone(domain='newzone.com')
        except ZoneAlreadyExistsError as e:
            self.assertEqual(e.zone_id, 'newzone.com')
        else:
            self.fail('Exception was not thrown')

    def test_get_record_record_does_not_exist(self):
        NsOneMockHttp.type = 'GET_RECORD_DOES_NOT_EXIST'

        try:
            self.driver.get_record(zone_id='getrecord.com', record_id='A')
        except RecordDoesNotExistError as e:
            self.assertEqual(e.record_id, 'A')
        else:
            self.fail('Exception was not thrown')

    def test_get_record_success(self):
        NsOneMockHttp.type = 'GET_RECORD_SUCCESS'
        record = self.driver.get_record(zone_id='getrecord.com', record_id='A')

        self.assertEqual(record.id, '520519509f782d58bb4df419')
        self.assertEqual(record.name, 'www.example.com')
        self.assertEqual(record.data, ['1.1.1.1'])
        self.assertEqual(record.type, RecordType.A)

    def test_list_records_zone_does_not_exist(self):
        NsOneMockHttp.type = 'LIST_RECORDS_ZONE_DOES_NOT_EXIST'

        try:
            self.driver.list_records(zone=self.test_zone)
        except ZoneDoesNotExistError as e:
            self.assertEqual(e.zone_id, self.test_zone.id)
        else:
            self.fail('Exception was not thrown')

    def test_list_records_empty(self):
        NsOneMockHttp.type = 'LIST_RECORDS_EMPTY'
        records = self.driver.list_records(zone=self.test_zone)

        self.assertEqual(records, [])

    def test_list_records_success(self):
        NsOneMockHttp.type = 'LIST_RECORDS_SUCCESS'
        records = self.driver.list_records(zone=self.test_zone)
        self.assertEqual(len(records), 2)

        arecord = records[1]
        self.assertEqual(arecord.id, '520519509f782d58bb4df419')
        self.assertEqual(arecord.name, 'www.example.com')
        self.assertEqual(arecord.type, RecordType.A)
        self.assertEqual(arecord.data, ['1.2.3.4'])

    def test_create_record_success(self):
        NsOneMockHttp.type = 'CREATE_RECORD_SUCCESS'
        arecord = self.driver.create_record(
            self.test_record.name,
            self.test_record.zone,
            self.test_record.type,
            self.test_record.data,
            self.test_record.extra,
        )
        self.assertEqual(arecord.id, '608f9619ebe68600ac9f807d')
        self.assertEqual(arecord.name, 'test.com')
        self.assertEqual(arecord.type, RecordType.A)
        self.assertEqual(arecord.data, ['127.0.0.1'])

    def test_create_record_already_exists(self):
        NsOneMockHttp.type = 'CREATE_RECORD_ALREADY_EXISTS'
        try:
            arecord = self.driver.create_record(
                self.test_record.name,
                self.test_record.zone,
                self.test_record.type,
                self.test_record.data,
                self.test_record.extra,
            )
        except RecordAlreadyExistsError as err:
            self.assertEqual(err.value, 'record already exists')
        else:
            self.fail('Exception was not thrown')

    def test_create_record_zone_not_found(self):
        NsOneMockHttp.type = 'CREATE_RECORD_ZONE_NOT_FOUND'
        try:
            arecord = self.driver.create_record(
                self.test_record.name,
                self.test_record.zone,
                self.test_record.type,
                self.test_record.data,
                self.test_record.extra,
            )
        except NsOneException as err:
            self.assertEqual(err.message, 'zone not found')
        else:
            self.fail('Exception was not thrown')

    def test_delete_record_record_does_not_exist(self):
        NsOneMockHttp.type = 'DELETE_RECORD_RECORD_DOES_NOT_EXIST'

        try:
            self.driver.delete_record(record=self.test_record)
        except RecordDoesNotExistError as e:
            self.assertEqual(e.record_id, self.test_record.id)
        else:
            self.fail('Exception was not thrown')

    def test_delete_record_success(self):
        NsOneMockHttp.type = 'DELETE_RECORD_SUCCESS'
        status = self.driver.delete_record(record=self.test_record)

        self.assertTrue(status)


class NsOneMockHttp(MockHttp):
    fixtures = DNSFileFixtures('nsone')

    def _v1_zones_EMPTY_ZONES_LIST(self, method, url, body, headers):
        body = self.fixtures.load('empty_zones_list.json')

        return httplib.OK, body, {}, httplib.responses[httplib.OK]

    def _v1_zones(self, method, url, body, headers):
        body = self.fixtures.load('list_zones.json')

        return httplib.OK, body, {}, httplib.responses[httplib.OK]

    def _v1_zones_getzone_com_GET_ZONE_SUCCESS(self, method, url, body,
                                               headers):
        body = self.fixtures.load('get_zone_success.json')

        return httplib.OK, body, {}, httplib.responses[httplib.OK]

    def _v1_zones_zonedoesnotexist_com_GET_ZONE_ZONE_DOES_NOT_EXIST(
            self, method, url, body, headers):
        body = self.fixtures.load('zone_does_not_exist.json')

        return 404, body, {}, httplib.responses[httplib.OK]

    def _v1_zones_test_com_DELETE_ZONE_SUCCESS(
            self, method, url, body, headers):
        body = self.fixtures.load('delete_zone_success.json')

        return httplib.OK, body, {}, httplib.responses[httplib.OK]

    def _v1_zones_test_com_DELETE_ZONE_ZONE_DOES_NOT_EXIST(
            self, method, url, body, headers):
        body = self.fixtures.load('zone_does_not_exist.json')

        return httplib.OK, body, {}, httplib.responses[httplib.OK]

    def _v1_zones_newzone_com_CREATE_ZONE_SUCCESS(self, method,
                                                  url, body, headers):
        body = self.fixtures.load('create_zone_success.json')

        return httplib.OK, body, {}, httplib.responses[httplib.OK]

    def _v1_zones_newzone_com_CREATE_ZONE_ZONE_ALREADY_EXISTS(
            self, method, url, body, headers):
        body = self.fixtures.load('zone_already_exists.json')

        return httplib.OK, body, {}, httplib.responses[httplib.OK]

    def _v1_zones_test_com_LIST_RECORDS_SUCCESS(
            self, method, url, body, headers):
        body = self.fixtures.load('get_zone_success.json')

        return httplib.OK, body, {}, httplib.responses[httplib.OK]

    def _v1_zones_test_com_LIST_RECORDS_EMPTY(
            self, method, url, body, headers):
        body = self.fixtures.load('list_records_empty.json')

        return httplib.OK, body, {}, httplib.responses[httplib.OK]

    def _v1_zones_test_com_LIST_RECORDS_ZONE_DOES_NOT_EXIST(
            self, method, url, body, headers):
        body = self.fixtures.load('zone_does_not_exist.json')

        return httplib.OK, body, {}, httplib.responses[httplib.OK]

    def _v1_zones_test_com_example_com_A_DELETE_RECORD_RECORD_DOES_NOT_EXIST(
            self, method, url, body, headers):
        body = self.fixtures.load('record_does_not_exist.json')

        return 404, body, {}, httplib.responses[httplib.OK]

    def _v1_zones_test_com_example_com_A_DELETE_RECORD_SUCCESS(
            self, method, url, body, headers):
        body = self.fixtures.load('delete_record_success.json')

        return httplib.OK, body, {}, httplib.responses[httplib.OK]

    def _v1_zones_getrecord_com_getrecord_com_A_GET_RECORD_SUCCESS(
            self, method, url, body, headers):
        body = self.fixtures.load('get_record_success.json')

        return httplib.OK, body, {}, httplib.responses[httplib.OK]

    def _v1_zones_getrecord_com_GET_RECORD_SUCCESS(
            self, method, url, body, headers):
        body = self.fixtures.load('get_zone_success.json')

        return httplib.OK, body, {}, httplib.responses[httplib.OK]

    def _v1_zones_getrecord_com_getrecord_com_A_GET_RECORD_DOES_NOT_EXIST(
            self, method, url, body, headers):
        body = self.fixtures.load('record_does_not_exist.json')

        return httplib.OK, body, {}, httplib.responses[httplib.OK]

    def _v1_zones_test_com_example_com_test_com_A_CREATE_RECORD_SUCCESS(
            self, method, url, body, headers):
        body = self.fixtures.load('create_record_success.json')

        return httplib.OK, body, {}, httplib.responses[httplib.OK]

    def _v1_zones_test_com_example_com_test_com_A_CREATE_RECORD_ALREADY_EXISTS(
            self, method, url, body, headers):
        body = self.fixtures.load('create_record_already_exists.json')

        return 404, body, {}, httplib.responses[httplib.OK]

    def _v1_zones_test_com_example_com_test_com_A_CREATE_RECORD_ZONE_NOT_FOUND(
            self, method, url, body, headers):
        body = self.fixtures.load('create_record_zone_not_found.json')

        return 404, body, {}, httplib.responses[httplib.OK]

if __name__ == '__main__':
    sys.exit(unittest.main())



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Common settings and connection objects for DigitalOcean Cloud
"""

from libcloud.utils.py3 import httplib, parse_qs, urlparse

from libcloud.common.base import BaseDriver
from libcloud.common.base import ConnectionKey
from libcloud.common.base import JsonResponse
from libcloud.common.types import LibcloudError, InvalidCredsError

__all__ = [
    'DigitalOcean_v2_Response',
    'DigitalOcean_v2_Connection',
    'DigitalOceanBaseDriver'
]


class DigitalOcean_v1_Error(LibcloudError):
    """
    Exception for when attempting to use version 1
    of the DigitalOcean API which is no longer
    supported.
    """

    def __init__(self,
                 value=('Driver no longer supported: Version 1 of the '
                        'DigitalOcean API reached end of life on November 9, '
                        '2015. Use the v2 driver. Please visit: '
                        'https://developers.digitalocean.com/documentation/changelog/api-v1/sunsetting-api-v1/'),  # noqa: E501
                 driver=None):
        super(DigitalOcean_v1_Error, self).__init__(value, driver=driver)


class DigitalOcean_v2_Response(JsonResponse):
    valid_response_codes = [httplib.OK, httplib.ACCEPTED, httplib.CREATED,
                            httplib.NO_CONTENT]

    def parse_error(self):
        if self.status == httplib.UNAUTHORIZED:
            body = self.parse_body()
            raise InvalidCredsError(body['message'])
        else:
            body = self.parse_body()
            if 'message' in body:
                error = '%s (code: %s)' % (body['message'], self.status)
            else:
                error = body
            return error

    def success(self):
        return self.status in self.valid_response_codes


class DigitalOcean_v2_Connection(ConnectionKey):
    """
    Connection class for the DigitalOcean (v2) driver.
    """

    host = 'api.digitalocean.com'
    responseCls = DigitalOcean_v2_Response

    def add_default_headers(self, headers):
        """
        Add headers that are necessary for every request

        This method adds ``token`` to the request.
        """
        headers['Authorization'] = 'Bearer %s' % (self.key)
        headers['Content-Type'] = 'application/json'
        return headers

    def add_default_params(self, params):
        """
        Add parameters that are necessary for every request

        This method adds ``per_page`` to the request to reduce the total
        number of paginated requests to the API.
        """
        # pylint: disable=maybe-no-member
        params['per_page'] = self.driver.ex_per_page
        return params


class DigitalOceanConnection(DigitalOcean_v2_Connection):
    """
    Connection class for the DigitalOcean driver.
    """
    pass


class DigitalOceanResponse(DigitalOcean_v2_Response):
    pass


class DigitalOceanBaseDriver(BaseDriver):
    """
    DigitalOcean BaseDriver
    """

    name = 'DigitalOcean'
    website = 'https://www.digitalocean.com'

    def __new__(cls, key, secret=None, api_version='v2', **kwargs):
        if cls is DigitalOceanBaseDriver:
            if api_version == 'v1' or secret is not None:
                raise DigitalOcean_v1_Error()
            elif api_version == 'v2':
                cls = DigitalOcean_v2_BaseDriver
            else:
                raise NotImplementedError('Unsupported API version: %s' %
                                          (api_version))
        return super(DigitalOceanBaseDriver, cls).__new__(cls, **kwargs)

    def ex_account_info(self):
        raise NotImplementedError(
            'ex_account_info not implemented for this driver')

    def ex_list_events(self):
        raise NotImplementedError(
            'ex_list_events not implemented for this driver')

    def ex_get_event(self, event_id):
        raise NotImplementedError(
            'ex_get_event not implemented for this driver')

    def _paginated_request(self, url, obj):
        raise NotImplementedError(
            '_paginated_requests not implemented for this driver')


class DigitalOcean_v2_BaseDriver(DigitalOceanBaseDriver):
    """
    DigitalOcean BaseDriver using v2 of the API.

    Supports `ex_per_page` ``int`` value keyword parameter to adjust per page
    requests against the API.
    """
    connectionCls = DigitalOcean_v2_Connection

    def __init__(self, key, secret=None, secure=True, host=None, port=None,
                 api_version=None, region=None, ex_per_page=200, **kwargs):
        self.ex_per_page = ex_per_page
        super(DigitalOcean_v2_BaseDriver, self).__init__(key, **kwargs)

    def ex_account_info(self):
        return self.connection.request('/v2/account').object['account']

    def ex_list_events(self):
        return self._paginated_request('/v2/actions', 'actions')

    def ex_get_event(self, event_id):
        """
        Get an event object

        :param      event_id: Event id (required)
        :type       event_id: ``str``
        """
        params = {}
        return self.connection.request('/v2/actions/%s' % event_id,
                                       params=params).object['action']

    def _paginated_request(self, url, obj):
        """
        Perform multiple calls in order to have a full list of elements when
        the API responses are paginated.

        :param url: API endpoint
        :type url: ``str``

        :param obj: Result object key
        :type obj: ``str``

        :return: ``list`` of API response objects
        :rtype: ``list``
        """
        params = {}
        data = self.connection.request(url)
        try:
            query = urlparse.urlparse(data.object['links']['pages']['last'])
            # The query[4] references the query parameters from the url
            pages = parse_qs(query[4])['page'][0]
            values = data.object[obj]
            for page in range(2, int(pages) + 1):
                params.update({'page': page})
                new_data = self.connection.request(url, params=params)

                more_values = new_data.object[obj]
                for value in more_values:
                    values.append(value)
            data = values
        except KeyError:  # No pages.
            data = data.object[obj]
        return data



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
NFSN DNS Driver
"""
import re

from libcloud.common.exceptions import BaseHTTPError
from libcloud.common.nfsn import NFSNConnection
from libcloud.dns.base import DNSDriver, Zone, Record
from libcloud.dns.types import ZoneDoesNotExistError, RecordDoesNotExistError
from libcloud.dns.types import RecordAlreadyExistsError
from libcloud.dns.types import Provider, RecordType
from libcloud.utils.py3 import httplib

__all__ = [
    'NFSNDNSDriver',
]

# The NFSN API does not return any internal "ID" strings for any DNS records.
# This means that we must set all returned Record objects' id properties to
# None. It also means that we cannot implement libcloud APIs that rely on
# record_id, such as get_record(). Instead, the NFSN-specific
# ex_get_records_by() method will return the desired Record objects.
#
# Additionally, the NFSN API does not provide ways to create, delete, or list
# all zones, so create_zone(), delete_zone(), and list_zones() are not
# implemented.


class NFSNDNSDriver(DNSDriver):
    type = Provider.NFSN
    name = 'NFSN DNS'
    website = 'https://www.nearlyfreespeech.net'
    connectionCls = NFSNConnection

    RECORD_TYPE_MAP = {
        RecordType.A: 'A',
        RecordType.AAAA: 'AAAA',
        RecordType.CNAME: 'CNAME',
        RecordType.MX: 'MX',
        RecordType.NS: 'NS',
        RecordType.SRV: 'SRV',
        RecordType.TXT: 'TXT',
        RecordType.PTR: 'PTR',
    }

    def list_records(self, zone):
        """
        Return a list of all records for the provided zone.

        :param zone: Zone to list records for.
        :type zone: :class:`Zone`

        :return: ``list`` of :class:`Record`
        """
        # Just use ex_get_records_by() with no name or type filters.
        return self.ex_get_records_by(zone)

    def get_zone(self, zone_id):
        """
        Return a Zone instance.

        :param zone_id: name of the required zone, for example "example.com".
        :type  zone_id: ``str``

        :rtype: :class:`Zone`
        :raises: ZoneDoesNotExistError: If no zone could be found.
        """
        # We will check if there is a serial property for this zone. If so,
        # then the zone exists.
        try:
            self.connection.request(action='/dns/%s/serial' % zone_id)
        except BaseHTTPError as e:
            if e.code == httplib.NOT_FOUND:
                raise ZoneDoesNotExistError(zone_id=None, driver=self,
                                            value=e.message)
            raise e
        return Zone(id=None, domain=zone_id, type='master', ttl=3600,
                    driver=self)

    def ex_get_records_by(self, zone, name=None, type=None):
        """
        Return a list of records for the provided zone, filtered by name and/or
        type.

        :param zone: Zone to list records for.
        :type zone: :class:`Zone`

        :param zone: Zone where the requested records are found.
        :type  zone: :class:`Zone`

        :param name: name of the records, for example "www". (optional)
        :type  name: ``str``

        :param type: DNS record type (A, MX, TXT). (optional)
        :type  type: :class:`RecordType`

        :return: ``list`` of :class:`Record`
        """
        payload = {}
        if name is not None:
            payload['name'] = name
        if type is not None:
            payload['type'] = type

        action = '/dns/%s/listRRs' % zone.domain
        response = self.connection.request(action=action, data=payload,
                                           method='POST')
        return self._to_records(response, zone)

    def create_record(self, name, zone, type, data, extra=None):
        """
        Create a new record.

        :param name: Record name without the domain name (e.g. www).
                     Note: If you want to create a record for a base domain
                     name, you should specify empty string ('') for this
                     argument.
        :type  name: ``str``

        :param zone: Zone where the requested record is created.
        :type  zone: :class:`Zone`

        :param type: DNS record type (A, MX, TXT).
        :type  type: :class:`RecordType`

        :param data: Data for the record (depends on the record type).
        :type  data: ``str``

        :param extra: Extra attributes (driver specific, e.g. 'ttl').
                      (optional)
        :type extra: ``dict``

        :rtype: :class:`Record`
        """
        action = '/dns/%s/addRR' % zone.domain
        payload = {'name': name, 'data': data, 'type': type}
        if extra is not None and extra.get('ttl', None) is not None:
            payload['ttl'] = extra['ttl']
        try:
            self.connection.request(action=action, data=payload, method='POST')
        except BaseHTTPError as e:
            exists_re = re.compile(r'That RR already exists on the domain')
            if e.code == httplib.BAD_REQUEST and \
               re.search(exists_re, e.message) is not None:
                value = '"%s" already exists in %s' % (name, zone.domain)
                raise RecordAlreadyExistsError(value=value, driver=self,
                                               record_id=None)
            raise e
        return self.ex_get_records_by(zone=zone, name=name, type=type)[0]

    def delete_record(self, record):
        """
        Use this method to delete a record.

        :param record: record to delete
        :type record: `Record`

        :rtype: Bool
        """
        action = '/dns/%s/removeRR' % record.zone.domain
        payload = {'name': record.name, 'data': record.data,
                   'type': record.type}
        try:
            self.connection.request(action=action, data=payload, method='POST')
        except BaseHTTPError as e:
            if e.code == httplib.NOT_FOUND:
                raise RecordDoesNotExistError(value=e.message, driver=self,
                                              record_id=None)
            raise e
        return True

    def _to_record(self, item, zone):
        ttl = int(item['ttl'])
        return Record(id=None, name=item['name'], data=item['data'],
                      type=item['type'], zone=zone, driver=self, ttl=ttl)

    def _to_records(self, items, zone):
        records = []
        for item in items.object:
            records.append(self._to_record(item, zone))
        return records



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from libcloud.common.google import GoogleOAuth2Credential
from libcloud.container.providers import Provider
from libcloud.container.drivers.kubernetes import KubernetesContainerDriver
from libcloud.common.google import GoogleResponse
from libcloud.common.google import GoogleBaseConnection
API_VERSION = 'v1'


class GKEResponse(GoogleResponse):
    pass


class GKEConnection(GoogleBaseConnection):
    """
    Connection class for the GKE driver.

    GKEConnection extends :class:`google.GoogleBaseConnection` for 3 reasons:
      1. modify request_path for GKE URI.
      2. Implement gce_params functionality described below.
      3. Add request_aggregated_items method for making aggregated API calls.

    """
    host = 'container.googleapis.com'
    responseCls = GKEResponse

    def __init__(self, user_id, key, secure, auth_type=None,
                 credential_file=None, project=None, **kwargs):
        super(GKEConnection, self).__init__(
            user_id, key, secure=secure, auth_type=auth_type,
            credential_file=credential_file, **kwargs)
        self.request_path = '/%s/projects/%s' % (API_VERSION, project)
        self.gke_params = {}

    def pre_connect_hook(self, params, headers):
        """
        Update URL parameters with values from self.gke_params.

        @inherits: :class:`GoogleBaseConnection.pre_connect_hook`
        """
        params, headers = super(GKEConnection, self).pre_connect_hook(params,
                                                                      headers)
        if self.gke_params:
            params.update(self.gke_params)
        return params, headers

    def request(self, *args, **kwargs):
        """
        Perform request then do GKE-specific processing of URL params.

        @inherits: :class:`GoogleBaseConnection.request`
        """
        response = super(GKEConnection, self).request(*args, **kwargs)

        # If gce_params has been set, then update the pageToken with the
        # nextPageToken so it can be used in the next request.
        if self.gke_params:
            if 'nextPageToken' in response.object:
                self.gke_params['pageToken'] = response.object['nextPageToken']
            elif 'pageToken' in self.gke_params:
                del self.gke_params['pageToken']
            self.gke_params = None

        return response


class GKEContainerDriver(KubernetesContainerDriver):
    """
    GKE Container Driver class.

    This is the primary driver for interacting with Google Container
    Engine. It contains all of the standard libcloud methods,
    plus additional ex_* methods for more features.

    Note that many methods allow either objects or strings (or lists of
    objects/strings).  In most cases, passing strings instead of objects
    will result in additional GKE API calls.
    """
    connectionCls = GKEConnection
    api_name = 'google'
    name = "Google Container Engine"
    type = Provider.GKE
    website = 'https://container.googleapis.com'
    supports_clusters = True

    AUTH_URL = "https://container.googleapis.com/auth/"

    def __init__(self, user_id, key=None, datacenter=None, project=None,
                 auth_type=None, scopes=None, credential_file=None,
                 host=None, port=443, **kwargs):
        """
        :param  user_id: The email address (for service accounts) or Client ID
                         (for installed apps) to be used for authentication.
        :type   user_id: ``str``

        :param  key: The RSA Key (for service accounts) or file path containing
                     key or Client Secret (for installed apps) to be used for
                     authentication.
        :type   key: ``str``

        :keyword  datacenter: The name of the datacenter (zone) used for
                              operations.
        :type     datacenter: ``str``

        :keyword  project: Your GKE project name. (required)
        :type     project: ``str``

        :keyword  auth_type: Accepted values are "SA" or "IA" or "GKE"
                             ("Service Account" or "Installed Application" or
                             "GKE" if libcloud is being used on a GKE instance
                             with service account enabled).
                             If not supplied, auth_type will be guessed based
                             on value of user_id or if the code is being
                             executed in a GKE instance.
        :type     auth_type: ``str``

        :keyword  scopes: List of authorization URLs. Default is empty and
                          grants read/write to Compute, Storage, DNS.
        :type     scopes: ``list``

        :keyword  credential_file: Path to file for caching authentication
                                   information used by GKEConnection.
        :type     credential_file: ``str``
        """
        if not project:
            raise ValueError('Project name must be specified using '
                             '"project" keyword.')
        if host is None:
            host = GKEContainerDriver.website
        self.auth_type = auth_type
        self.project = project
        self.scopes = scopes
        self.zone = None
        if datacenter is not None:
            self.zone = datacenter
        self.credential_file = credential_file or \
            GoogleOAuth2Credential.default_credential_file + '.' + self.project

        super(GKEContainerDriver, self).__init__(user_id, key,
                                                 secure=True, host=None,
                                                 port=None, **kwargs)

        self.base_path = '/%s/projects/%s' % (API_VERSION, self.project)
        self.website = GKEContainerDriver.website

    def _ex_connection_class_kwargs(self):
        return {'auth_type': self.auth_type,
                'project': self.project,
                'scopes': self.scopes,
                'credential_file': self.credential_file}

    def list_clusters(self, ex_zone=None):
        """
        Return a list of cluster information in the current zone or all zones.

        :keyword  ex_zone:  Optional zone name or None
        :type     ex_zone:  ``str`` or :class:`GCEZone` or
                            :class:`NodeLocation` or ``None``
        """
        request = "/zones/%s/clusters" % (ex_zone)
        if ex_zone is None:
            request = "/zones/clusters"

        response = self.connection.request(request, method='GET').object
        return response

    def get_server_config(self, ex_zone=None):
        """
        Return configuration info about the Container Engine service.

        :keyword  ex_zone:  Optional zone name or None
        :type     ex_zone:  ``str`` or :class:`GCEZone` or
                            :class:`NodeLocation` or ``None``
        """
        if ex_zone is None:
            ex_zone = self.zone
        request = "/zones/%s/serverconfig" % (ex_zone)

        response = self.connection.request(request, method='GET').object
        return response



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from libcloud.compute.types import Provider as NodeProvider
from libcloud.compute.providers import get_driver as get_node_driver
from libcloud.loadbalancer.providers import get_driver
from libcloud.loadbalancer.base import Algorithm, Member
from libcloud.loadbalancer.types import Provider

SLBDriver = get_driver(Provider.ALIYUN_SLB)
ECSDriver = get_node_driver(NodeProvider.ALIYUN_ECS)

region = 'cn-hangzhou'

your_access_key_id = ''
your_access_key_secret = ''
slb = SLBDriver(your_access_key_id, your_access_key_secret, region=region)
ecs = ECSDriver(your_access_key_id, your_access_key_secret, region=region)

protos = slb.list_protocols()
print('Found %d protocols: %s' % (len(protos), protos))

balancers = slb.list_balancers()
print('Found %d load balancers' % len(balancers))
print(balancers)

if len(balancers) > 0:
    b1 = balancers[0]
    print('Delete %s' % b1)
    slb.destroy_balancer(b1)
else:
    extra = {'AddressType': 'internet',
             'Bandwidth': 1,
             'StickySession': 'off',
             'HealthCheck': 'off'}
    nodes = ecs.list_nodes()
    print('Found %d nodes' % len(nodes))
    members = [Member(node.id, node.public_ips[0], 80,
                      extra={'Weight': 50 * (i + 1)})
               for i, node in enumerate(nodes)]
    new_b = slb.create_balancer('test-balancer', 80, 'http',
                                Algorithm.WEIGHTED_ROUND_ROBIN, members,
                                **extra)
    print('Created balancer %s' % new_b)
